@@ -28,14 +28,14 @@ import struct
 import datetime
 from decimal import Decimal
 
-from mysql.connector.constants import (
+from .constants import (
     FieldFlag, ServerCmd, FieldType, ClientFlag)
-from mysql.connector import (errors, utils)
+from . import errors, utils
 from .authentication import get_auth_plugin
+from .catch23 import PY2, struct_unpack
 
 
 class MySQLProtocol(object):
-
     """Implements MySQL client/server protocol
 
     Create and parses MySQL packets.
@@ -44,19 +44,18 @@ class MySQLProtocol(object):
     def _connect_with_db(self, client_flags, database):
         """Prepare database string for handshake response"""
         if client_flags & ClientFlag.CONNECT_WITH_DB and database:
-            if isinstance(database, unicode):
-                return database.encode('utf8') + '\x00'
-            return database + '\x00'
-        return '\x00'
+            return database.encode('utf8') + b'\x00'
+        return b'\x00'
 
     def _auth_response(self, client_flags, username, password, database,
                        auth_plugin, auth_data, ssl_enabled):
         """Prepare the authentication response"""
         if not password:
-            return '\x00'
+            return b'\x00'
 
         try:
-            auth = get_auth_plugin(auth_plugin)(auth_data,
+            auth = get_auth_plugin(auth_plugin)(
+                auth_data,
                 username=username, password=password, database=database,
                 ssl_enabled=ssl_enabled)
             plugin_auth_response = auth.auth_response()
@@ -68,7 +67,7 @@ class MySQLProtocol(object):
             resplen = len(plugin_auth_response)
             auth_response = struct.pack('<B', resplen) + plugin_auth_response
         else:
-            auth_response = plugin_auth_response + '\x00'
+            auth_response = plugin_auth_response + b'\x00'
         return auth_response
 
     def make_auth(self, handshake, username=None, password=None, database=None,
@@ -85,12 +84,16 @@ class MySQLProtocol(object):
                 "Handshake misses authentication info ({0})".format(exc))
 
         if not username:
-            username = ''
-        elif isinstance(username, unicode):
-            username = username.encode('utf8')
+            username = b''
+        try:
+            username_bytes = username.encode('utf8')  # pylint: disable=E1103
+        except AttributeError:
+            # Username is already bytes
+            username_bytes = username
         packet = struct.pack('<IIB{filler}{usrlen}sx'.format(
-                filler='x'*23, usrlen=len(username)),
-            client_flags, max_allowed_packet, charset, username)
+            filler='x' * 23, usrlen=len(username_bytes)),
+                             client_flags, max_allowed_packet, charset,
+                             username_bytes)
 
         packet += self._auth_response(client_flags, username, password,
                                       database,
@@ -100,23 +103,23 @@ class MySQLProtocol(object):
         packet += self._connect_with_db(client_flags, database)
 
         if client_flags & ClientFlag.PLUGIN_AUTH:
-            packet += auth_plugin.encode('utf8') + '\x00'
+            packet += auth_plugin.encode('utf8') + b'\x00'
 
         return packet
 
     def make_auth_ssl(self, charset=33, client_flags=0,
                       max_allowed_packet=1073741824):
         """Make a SSL authentication packet"""
-        return utils.int4store(client_flags) +\
-               utils.int4store(max_allowed_packet) +\
-               utils.int1store(charset) +\
-               '\x00' * 23
+        return utils.int4store(client_flags) + \
+               utils.int4store(max_allowed_packet) + \
+               utils.int1store(charset) + \
+               b'\x00' * 23
 
     def make_command(self, command, argument=None):
         """Make a MySQL packet containing a command"""
         data = utils.int1store(command)
         if argument is not None:
-            data += str(argument)
+            data += argument
         return data
 
     def make_change_user(self, handshake, username=None, password=None,
@@ -132,11 +135,14 @@ class MySQLProtocol(object):
                 "Handshake misses authentication info ({0})".format(exc))
 
         if not username:
-            username = ''
-        elif isinstance(username, unicode):
-            username = username.encode('utf8')
-        packet = struct.pack('<B{usrlen}sx'.format(usrlen=len(username)),
-            ServerCmd.CHANGE_USER, username)
+            username = b''
+        try:
+            username_bytes = username.encode('utf8')  # pylint: disable=E1103
+        except AttributeError:
+            # Username is already bytes
+            username_bytes = username
+        packet = struct.pack('<B{usrlen}sx'.format(usrlen=len(username_bytes)),
+                             ServerCmd.CHANGE_USER, username_bytes)
 
         packet += self._auth_response(client_flags, username, password,
                                       database,
@@ -148,16 +154,16 @@ class MySQLProtocol(object):
         packet += struct.pack('<H', charset)
 
         if client_flags & ClientFlag.PLUGIN_AUTH:
-            packet += auth_plugin.encode('utf8') + '\x00'
+            packet += auth_plugin.encode('utf8') + b'\x00'
 
         return packet
 
     def parse_handshake(self, packet):
         """Parse a MySQL Handshake-packet"""
         res = {}
-        res['protocol'] = struct.unpack('<xxxxB', packet[0:5])[0]
+        res['protocol'] = struct_unpack('<xxxxB', packet[0:5])[0]
         (packet, res['server_version_original']) = utils.read_string(
-            packet[5:], end='\x00')
+            packet[5:], end=b'\x00')
 
         (res['server_threadid'],
          auth_data1,
@@ -166,23 +172,23 @@ class MySQLProtocol(object):
          res['server_status'],
          capabilities2,
          auth_data_length
-         ) = struct.unpack('<I8sx2sBH2sBxxxxxxxxxx', packet[0:31])
+        ) = struct_unpack('<I8sx2sBH2sBxxxxxxxxxx', packet[0:31])
 
         packet = packet[31:]
 
         capabilities = utils.intread(capabilities1 + capabilities2)
-
-        auth_data2 = ''
+        auth_data2 = b''
         if capabilities & ClientFlag.SECURE_CONNECTION:
             size = min(13, auth_data_length - 8) if auth_data_length else 13
             auth_data2 = packet[0:size]
             packet = packet[size:]
-            if auth_data2[-1] == '\x00':
+            if auth_data2[-1] == 0:
                 auth_data2 = auth_data2[:-1]
 
         if capabilities & ClientFlag.PLUGIN_AUTH:
             (packet, res['auth_plugin']) = utils.read_string(
-                packet, end='\x00')
+                packet, end=b'\x00')
+            res['auth_plugin'] = res['auth_plugin'].decode('utf-8')
         else:
             res['auth_plugin'] = 'mysql_native_password'
 
@@ -192,19 +198,20 @@ class MySQLProtocol(object):
 
     def parse_ok(self, packet):
         """Parse a MySQL OK-packet"""
-        if not packet[4] == '\x00':
+        if not packet[4] == 0:
             raise errors.InterfaceError("Failed parsing OK packet.")
 
         ok_packet = {}
         try:
-            ok_packet['field_count'] = struct.unpack('<xxxxB', packet[0:5])[0]
+            ok_packet['field_count'] = struct_unpack('<xxxxB', packet[0:5])[0]
             (packet, ok_packet['affected_rows']) = utils.read_lc_int(packet[5:])
             (packet, ok_packet['insert_id']) = utils.read_lc_int(packet)
             (ok_packet['server_status'],
-                ok_packet['warning_count']) = struct.unpack('<HH', packet[0:4])
+             ok_packet['warning_count']) = struct_unpack('<HH', packet[0:4])
             packet = packet[4:]
             if packet:
                 (packet, ok_packet['info_msg']) = utils.read_lc_string(packet)
+                ok_packet['info_msg'] = ok_packet['info_msg'].decode('utf-8')
         except ValueError:
             raise errors.InterfaceError("Failed parsing OK packet.")
         return ok_packet
@@ -218,7 +225,7 @@ class MySQLProtocol(object):
 
     def parse_column(self, packet):
         """Parse a MySQL column-packet"""
-        (packet, _) = utils.read_lc_string(packet[4:]) # catalog
+        (packet, _) = utils.read_lc_string(packet[4:])  # catalog
         (packet, _) = utils.read_lc_string(packet)  # db
         (packet, _) = utils.read_lc_string(packet)  # table
         (packet, _) = utils.read_lc_string(packet)  # org_table
@@ -227,12 +234,12 @@ class MySQLProtocol(object):
 
         try:
             (_, _, field_type,
-             flags, _) = struct.unpack('<xHIBHBxx', packet)
+             flags, _) = struct_unpack('<xHIBHBxx', packet)
         except struct.error:
             raise errors.InterfaceError("Failed parsing column information")
 
         return (
-            name,
+            name.decode('utf-8'),
             field_type,
             None,  # display_size
             None,  # internal_size
@@ -240,14 +247,14 @@ class MySQLProtocol(object):
             None,  # scale
             ~flags & FieldFlag.NOT_NULL,  # null_ok
             flags,  # MySQL specific
-            )
+        )
 
     def parse_eof(self, packet):
         """Parse a MySQL EOF-packet"""
         err_msg = "Failed parsing EOF packet."
         res = {}
         try:
-            unpacked = struct.unpack('<xxxBBHH', packet)
+            unpacked = struct_unpack('<xxxBBHH', packet)
         except struct.error:
             raise errors.InterfaceError(err_msg)
 
@@ -262,23 +269,24 @@ class MySQLProtocol(object):
         """Parse the statistics packet"""
         errmsg = "Failed getting COM_STATISTICS information"
         res = {}
-         # Information is separated by 2 spaces
-        pairs = packet[4:].split('\x20\x20')
+        # Information is separated by 2 spaces
+        pairs = packet[4:].split(b'\x20\x20')
         for pair in pairs:
             try:
-                (lbl, val) = [v.strip() for v in pair.split(':', 2)]
+                (lbl, val) = [v.strip() for v in pair.split(b':', 2)]
             except:
                 raise errors.InterfaceError(errmsg)
 
             # It's either an integer or a decimal
+            lbl = lbl.decode('utf-8')
             try:
-                res[lbl] = long(val)
+                res[lbl] = int(val)
             except:
                 try:
-                    res[lbl] = Decimal(val)
+                    res[lbl] = Decimal(val.decode('utf-8'))
                 except:
                     raise errors.InterfaceError(
-                        "%s (%s:%s)." % (errmsg, lbl, val))
+                        "{0} ({1}:{2}).".format(errmsg, lbl, val))
         return res
 
     def read_text_result(self, sock, count=1):
@@ -299,18 +307,18 @@ class MySQLProtocol(object):
             if i == count:
                 break
             packet = sock.recv()
-            if packet.startswith('\xff\xff\xff'):
+            if packet.startswith(b'\xff\xff\xff'):
                 datas = [packet[4:]]
                 packet = sock.recv()
-                while packet.startswith('\xff\xff\xff'):
+                while packet.startswith(b'\xff\xff\xff'):
                     datas.append(packet[4:])
                     packet = sock.recv()
-                if packet[4] == '\xfe':
+                if packet[4] == 254:
                     eof = self.parse_eof(packet)
                 else:
                     datas.append(packet[4:])
-                rowdata = utils.read_lc_string_list(''.join(datas))
-            elif packet[4] == '\xfe':
+                rowdata = utils.read_lc_string_list(b''.join(datas))
+            elif packet[4] == 254:
                 eof = self.parse_eof(packet)
                 rowdata = None
             else:
@@ -339,7 +347,7 @@ class MySQLProtocol(object):
         if field[7] & FieldFlag.UNSIGNED:
             format_ = format_.upper()
 
-        return (packet[length:], struct.unpack(format_, packet[0:length])[0])
+        return (packet[length:], struct_unpack(format_, packet[0:length])[0])
 
     def _parse_binary_float(self, packet, field):
         """Parse a float/double from a binary packet"""
@@ -350,59 +358,59 @@ class MySQLProtocol(object):
             length = 4
             format_ = 'f'
 
-        return (packet[length:], struct.unpack(format_, packet[0:length])[0])
+        return (packet[length:], struct_unpack(format_, packet[0:length])[0])
 
     def _parse_binary_timestamp(self, packet, field):
         """Parse a timestamp from a binary packet"""
-        length = ord(packet[0])
+        length = packet[0]
         value = None
         if length == 4:
             value = datetime.date(
-                year=struct.unpack('H', packet[1:3])[0],
-                month=ord(packet[3]),
-                day=ord(packet[4]))
+                year=struct_unpack('H', packet[1:3])[0],
+                month=packet[3],
+                day=packet[4])
         elif length >= 7:
             mcs = 0
             if length == 11:
-                mcs = struct.unpack('I', packet[8:length + 1])[0]
+                mcs = struct_unpack('I', packet[8:length + 1])[0]
             value = datetime.datetime(
-                year=struct.unpack('H', packet[1:3])[0],
-                month=ord(packet[3]),
-                day=ord(packet[4]),
-                hour=ord(packet[5]),
-                minute=ord(packet[6]),
-                second=ord(packet[7]),
+                year=struct_unpack('H', packet[1:3])[0],
+                month=packet[3],
+                day=packet[4],
+                hour=packet[5],
+                minute=packet[6],
+                second=packet[7],
                 microsecond=mcs)
 
         return (packet[length + 1:], value)
 
     def _parse_binary_time(self, packet, field):
         """Parse a time value from a binary packet"""
-        length = ord(packet[0])
+        length = packet[0]
         data = packet[1:length + 1]
         mcs = 0
         if length > 8:
-            mcs = struct.unpack('I', data[8:])[0]
-        days = struct.unpack('I', data[1:5])[0]
-        if int(ord(data[0])) == 1:
+            mcs = struct_unpack('I', data[8:])[0]
+        days = struct_unpack('I', data[1:5])[0]
+        if data[0] == 1:
             days *= -1
         tmp = datetime.timedelta(days=days,
-                                 seconds=int(ord(data[7])),
+                                 seconds=data[7],
                                  microseconds=mcs,
-                                 minutes=int(ord(data[6])),
-                                 hours=int(ord(data[5])))
+                                 minutes=data[6],
+                                 hours=data[5])
 
         return (packet[length + 1:], tmp)
 
     def _parse_binary_values(self, fields, packet):
         """Parse values from a binary result packet"""
         null_bitmap_length = (len(fields) + 7 + 2) // 8
-        null_bitmap = utils.intread(packet[0:null_bitmap_length])
+        null_bitmap = [int(i) for i in packet[0:null_bitmap_length]]
         packet = packet[null_bitmap_length:]
 
         values = []
         for pos, field in enumerate(fields):
-            if null_bitmap & 1 << (pos + 2):
+            if null_bitmap[int((pos+2)/8)] & (1 << (pos + 2) % 8):
                 values.append(None)
                 continue
             elif field[1] in (FieldType.TINY, FieldType.SHORT,
@@ -441,10 +449,10 @@ class MySQLProtocol(object):
             if i == count:
                 break
             packet = sock.recv()
-            if packet[4] == '\xfe':
+            if packet[4] == 254:
                 eof = self.parse_eof(packet)
                 values = None
-            elif packet[4] == '\x00':
+            elif packet[4] == 0:
                 eof = None
                 values = self._parse_binary_values(columns, packet[5:])
             if eof is None and values is not None:
@@ -454,20 +462,20 @@ class MySQLProtocol(object):
 
     def parse_binary_prepare_ok(self, packet):
         """Parse a MySQL Binary Protocol OK packet"""
-        if not packet[4] == '\x00':
+        if not packet[4] == 0:
             raise errors.InterfaceError("Failed parsing Binary OK packet")
 
-        ok_packet = {}
+        ok_pkt = {}
         try:
-            (packet, ok_packet['statement_id']) = utils.read_int(packet[5:], 4)
-            (packet, ok_packet['num_columns']) = utils.read_int(packet, 2)
-            (packet, ok_packet['num_params']) = utils.read_int(packet, 2)
+            (packet, ok_pkt['statement_id']) = utils.read_int(packet[5:], 4)
+            (packet, ok_pkt['num_columns']) = utils.read_int(packet, 2)
+            (packet, ok_pkt['num_params']) = utils.read_int(packet, 2)
             packet = packet[1:]  # Filler 1 * \x00
-            (packet, ok_packet['warning_count']) = utils.read_int(packet, 2)
+            (packet, ok_pkt['warning_count']) = utils.read_int(packet, 2)
         except ValueError:
             raise errors.InterfaceError("Failed parsing Binary OK packet")
 
-        return ok_packet
+        return ok_pkt
 
     def _prepare_binary_integer(self, value):
         """Prepare an integer for the MySQL binary protocol"""
@@ -555,7 +563,7 @@ class MySQLProtocol(object):
         field_type = FieldType.TIME
         negative = 0
         mcs = None
-        packed = ''
+        packed = b''
 
         if isinstance(value, datetime.timedelta):
             if value.days < 0:
@@ -599,7 +607,7 @@ class MySQLProtocol(object):
         null_bitmap = [0] * ((len(data) + 7) // 8)
         values = []
         types = []
-        packed = ''
+        packed = b''
         if long_data_used is None:
             long_data_used = {}
         if parameters and data:
@@ -612,6 +620,8 @@ class MySQLProtocol(object):
                 flags = 0
                 if value is None:
                     null_bitmap[(pos // 8)] |= 1 << (pos % 8)
+                    types.append(utils.int1store(FieldType.NULL) +
+                                 utils.int1store(flags))
                     continue
                 elif pos in long_data_used:
                     if long_data_used[pos][0]:
@@ -620,19 +630,31 @@ class MySQLProtocol(object):
                     else:
                         # We suppose text data
                         field_type = FieldType.STRING
-                elif isinstance(value, (int, long)):
+                elif isinstance(value, int):
                     (packed, field_type,
                      flags) = self._prepare_binary_integer(value)
                     values.append(packed)
                 elif isinstance(value, str):
-                    values.append(utils.intstore(len(value)) + value)
+                    if PY2:
+                        values.append(utils.lc_int(len(value)) +
+                                      value)
+                    else:
+                        value = value.encode(charset)
+                        values.append(
+                            utils.lc_int(len(value)) + value)
                     field_type = FieldType.VARCHAR
-                elif isinstance(value, unicode):
+                elif isinstance(value, bytes):
+                    values.append(utils.lc_int(len(value)) + value)
+                    field_type = FieldType.BLOB
+                elif PY2 and \
+                        isinstance(value, unicode):  # pylint: disable=E0602
                     value = value.encode(charset)
-                    values.append(utils.intstore(len(value)) + value)
+                    values.append(utils.lc_int(len(value)) + value)
                     field_type = FieldType.VARCHAR
                 elif isinstance(value, Decimal):
-                    values.append(utils.intstore(len(str(value))) + str(value))
+                    values.append(
+                        utils.lc_int(len(str(value).encode(
+                            charset))) + str(value).encode(charset))
                     field_type = FieldType.DECIMAL
                 elif isinstance(value, float):
                     values.append(struct.pack('d', value))
@@ -653,31 +675,36 @@ class MySQLProtocol(object):
                              utils.int1store(flags))
 
         packet = (
-            utils.int4store(statement_id),
-            utils.int1store(flags),
-            utils.int4store(iteration_count),
-            ''.join([struct.pack('B', bit) for bit in null_bitmap]),
-            utils.int1store(1),
-            ''.join(types),
-            ''.join(values)
-            )
-        return ''.join(packet)
+            utils.int4store(statement_id) +
+            utils.int1store(flags) +
+            utils.int4store(iteration_count) +
+            b''.join([struct.pack('B', bit) for bit in null_bitmap]) +
+            utils.int1store(1)
+        )
+
+        for a_type in types:
+            packet += a_type
+
+        for a_value in values:
+            packet += a_value
+
+        return packet
 
     def parse_auth_switch_request(self, packet):
         """Parse a MySQL AuthSwitchRequest-packet"""
-        if not packet[4] == '\xfe':
+        if not packet[4] == 254:
             raise errors.InterfaceError(
                 "Failed parsing AuthSwitchRequest packet")
 
-        (packet, plugin_name) = utils.read_string(packet[5:], end='\x00')
-        if packet[-1] == '\x00':
+        (packet, plugin_name) = utils.read_string(packet[5:], end=b'\x00')
+        if packet[-1] == 0:
             packet = packet[:-1]
 
-        return plugin_name, packet
+        return plugin_name.decode('utf8'), packet
 
     def parse_auth_more_data(self, packet):
         """Parse a MySQL AuthMoreData-packet"""
-        if not packet[4] == '\x01':
+        if not packet[4] == 1:
             raise errors.InterfaceError(
                 "Failed parsing AuthMoreData packet")
 
