@@ -40,17 +40,17 @@ import re
 import inspect
 import logging
 import functools
-import traceback
+import collections
+import json
 
 import mysql.fabric.errors as _errors
 import mysql.fabric.executor as _executor
+import mysql.fabric.utils as _utils
 
-from mysql.fabric.sharding import (
-    MappingShardsGroups,
-)
+from cStringIO import StringIO
 
-from mysql.fabric import (
-    persistence as _persistence,
+from mysql.fabric.utils import (
+    FABRIC_UUID,
 )
 
 from mysql.fabric.handler import (
@@ -156,7 +156,6 @@ class CommandMeta(type):
             group = obj.group_name
             command = obj.command_name
             subject = ".".join([group, command])
-            success = True
             try:
                 _LOGGER.debug(
                     "Started command (%s, %s).", group, command,
@@ -167,18 +166,31 @@ class CommandMeta(type):
                     }
                 )
                 ret = original(obj, *args, **kwrds)
-                if isinstance(obj, ProcedureCommand):
-                    success = ProcedureCommand.succeeded(ret)
-            except:
-                success = False
-                raise
+
+                # Check that we really got a result set back. If not,
+                # something is amiss.
+                #
+                # As a special case, if the function returns None it
+                # means it finished without throwing an exception, so
+                # it trivially succeeded without a result set.
+                if ret is None:
+                    ret = CommandResult(None)
+                elif not isinstance(ret, CommandResult):
+                    raise  _errors.InternalError(
+                        "Expected '%s', got '%s'" % (
+                            CommandResult.__name__,
+                            ret,
+                        )
+                    )
+            except Exception as error:
+                ret = CommandResult(error=str(error))
             finally:
                 _LOGGER.debug("Finished command (%s, %s).", group, command,
                     extra={
                         "subject" : subject,
                         "category" : MySQLHandler.PROCEDURE,
-                        "type" : MySQLHandler.STOP if success else \
-                                 MySQLHandler.ABORT
+                        "type" : MySQLHandler.ABORT if ret.error else \
+                                 MySQLHandler.STOP
                     }
                 )
             return ret
@@ -239,23 +251,35 @@ class Command(object):
 
     command_name = None
 
-    command_options = []
-
     def __init__(self):
+        """Constructor.
+
+        See class description for information.
+        """
         self.__client = None
         self.__server = None
         self.__options = None
         self.__config = None
-        #use the execute / dispatch method signature to build the
-        #optional argument list passed to the command line parser.
+
+        try:
+            self.command_options
+        except AttributeError:
+            self.command_options = []
+
+        self.generate_options()
+
+    def generate_options(self):
+        """Use the execute / dispatch method signature to build the
+        optional argument list passed to the command line parser.
+        """
+        # Extract the default values from the method signature and build
+        # the optional argument list.
         try:
             spec = inspect.getargspec(self.execute.original_function)
         except AttributeError:
             spec = inspect.getargspec(self.dispatch)
-        #Extract the default values from the method signature and build
-        #the optional argument list.
+
         if spec.defaults is not None:
-            self.command_options = []
             action = ""
             #Easier to build the default args and values pairs in reverse
             for opt, value in zip(reversed(spec.args), reversed(spec.defaults)):
@@ -389,48 +413,7 @@ class Command(object):
 
         :param args: The arguments for the command dispatch.
         """
-        status = self.client.dispatch(self, *args)
-        return self.command_status(status)
-
-    def execute(self):
-        """Any command derived from this class must redefine this
-        method.
-        """
-        raise _errors.ProgrammingError("The execute method is not defined.")
-
-    @staticmethod
-    def command_status(status, details=False):
-        """Present the result reported by a command in a friendly-user way.
-
-        :param status: The command status.
-        :param details: Whether details on failures should be printed or
-                        not.
-        """
-        success = True
-        returned = ""
-        activities = ""
-
-        if not isinstance(status, list):
-            returned = status
-        elif not isinstance(status[0], bool):
-            returned = status
-        else:
-            success = status[0]
-            if success:
-                returned = status[2]
-            else:
-                trace = status[1].split("\n")
-                returned = trace[-2]
-                if details:
-                    activities = "\n".join(trace)
-
-        return "\n".join([
-            "Command :",
-            "{ success     = %s" % (success, ),
-            "  return      = %s" % (returned, ),
-            "  activities  = %s" % (activities, ),
-            "}"
-            ])
+        return self.client.dispatch(self, *args)
 
     @classmethod
     def get_signature(cls):
@@ -494,14 +477,24 @@ class Command(object):
         :param func: the function that needs to be called
         :param params: The parameters to the function
 
-        :return: {success:True/False, message:<for example exception>,
-                return:<return values>}.
+        :return: :class:`CommandResult` instance
+
         """
-        try:
-            status = func(*params)
-        except Exception:
-            return [False, traceback.format_exc(), True]
-        return [True, "", status]
+
+        status = func(*params)
+        _LOGGER.debug(
+            "Status from execution of '%s': %s", func.__name__, status
+        )
+        if len(status) > 0:
+            rset = ResultSet(
+                names=status[0].keys(),
+                types=[type(v) for v in status[0].values()],
+            )
+            for entry in status:
+                rset.append_row(entry.values())
+        else:
+            rset = None
+        return CommandResult(None, results=rset)
 
 
 class ProcedureCommand(Command):
@@ -537,8 +530,8 @@ class ProcedureCommand(Command):
 
         :param args: The arguments for the command dispatch.
         """
-        status = self.client.dispatch(self, *args)
-        return self.procedure_status(status)
+
+        return self.client.dispatch(self, *args)
 
     @staticmethod
     def wait_for_procedures(procedure_param, synchronous):
@@ -552,82 +545,65 @@ class ProcedureCommand(Command):
         :param procedure_param: Iterable with procedures.
         :param synchronous: Whether should wait until the procedure
                             finishes its execution or not.
-        :return: Information on the procedure.
-        :rtype: str(procedure.uuid), procedure.status, procedure.result
-                or (str(procedure.uuid))
+        :return: A :class:`CommandResult` instance with the execution result.
+        :rtype: CommandResult
         """
-        assert(len(procedure_param) == 1)
+        assert len(procedure_param) == 1
         synchronous = str(synchronous).upper() not in ("FALSE", "0")
-        if synchronous:
-            executor = _executor.Executor()
-            for procedure in procedure_param:
-                executor.wait_for_procedure(procedure)
-            return str(procedure_param[-1].uuid), procedure_param[-1].status, \
-                procedure_param[-1].result
-        else:
-            return str(procedure_param[-1].uuid)
+        if not synchronous:
+            info = ResultSet(names=['uuid'], types=[str])
+            info.append_row([str(procedure_param[-1].uuid)])
+            return CommandResult(None, results=info)
 
-    @staticmethod
-    def procedure_status(status, details=False):
-        """Transform a status reported by :func:`wait_for_procedures` into
-        a string that can be used by the command-line interface.
+        executor = _executor.Executor()
+        for procedure in procedure_param:
+            executor.wait_for_procedure(procedure)
+        _LOGGER.debug(
+            "Result after wait: uuid='%s', status='%s', result='%s'",
+            str(procedure_param[-1].uuid), procedure_param[-1].status,
+            procedure_param[-1].result
+        )
 
-        :param status: The status of the command execution.
-        :param details: Boolean that indicates if detailed execution status
-                        be returned.
+        # We look at the diagnosis of the last entry to decide the
+        # status of the procedure execution.
+        operation = procedure_param[-1].status[-1]
+        complete = operation['state'] == _executor.Job.COMPLETE
+        success = operation['success'] == _executor.Job.SUCCESS
 
-        :return: Return the detailed execution status as a string.
-        """
-        proc_id = None
-        complete = ""
-        success = ""
-        returned = ""
-        activities = ""
-
-        if isinstance(status, str):
-            proc_id = status
-        else:
-            proc_id = status[0]
-            operation = status[1][-1]
-            returned = None
-            activities = ""
-            complete = (operation["state"] == _executor.Job.COMPLETE)
-            success = (operation["success"] == _executor.Job.SUCCESS)
-
-            if success:
-                returned = status[2]
-                if details:
-                    steps = [step["description"] for step in status[1]]
-                    activities = "\n  ".join(steps)
-            else:
-                trace = operation["diagnosis"].split("\n")
-                returned = trace[-2]
-                if details:
-                    activities = "\n".join(trace)
-
-        return "\n".join([
-            "Procedure :",
-            "{ uuid        = %s," % (proc_id, ),
-            "  finished    = %s," % (complete, ),
-            "  success     = %s," % (success, ),
-            "  return      = %s," % (returned, ),
-            "  activities  = %s"  % (activities, ),
-            "}"
+        if success:
+            result_field = procedure_param[-1].result
+            info = ResultSet(
+                names=('uuid', 'finished', 'success', 'result'),
+                types=(str, bool, bool, type(result_field)),
+            )
+            info.append_row([
+                str(procedure_param[-1].uuid),
+                complete,
+                success,
+                result_field,
             ])
+            _LOGGER.debug("Success: uuid='%s', result='%s'",
+                          str(procedure_param[-1].uuid),
+                          str(procedure_param[-1].result))
 
-    @staticmethod
-    def succeeded(status):
-        """Check whether a procedure has succeeded or not.
-        """
-        ret = True
-
-        try:
-            ret = (status[1][-1]["success"] == _executor.Job.SUCCESS)
-        except TypeError:
-            # This may happen if the procedure is asynchronously executed.
-            pass
-
-        return ret
+            rset = ResultSet(
+                names=('state', 'success', 'when', 'description'),
+                types=(int, int, float, str),
+            )
+            for item in procedure_param[-1].status:
+                rset.append_row([
+                    item['state'],
+                    item['success'],
+                    item['when'],
+                    item['description'],
+                ])
+            return CommandResult(None, results=[info, rset])
+        else:
+            # The error message is the last line of the diagnosis, so
+            # we get it from there.
+            error = operation['diagnosis'].split("\n")[-2]
+            _LOGGER.debug("Failure: error='%s'", error)
+            return CommandResult(error)
 
     def get_lockable_objects(self, variable=None, function=None):
         """Return the set of lockable objects by extracting information
@@ -656,3 +632,311 @@ class ProcedureShard(ProcedureCommand):
     execute operations within a sharding.
     """
     pass
+
+ResultSetColumn = collections.namedtuple('ResultSetColumn', 'name,type')
+
+class ResultSet(object):
+    """A result set returned by a command object.
+
+    Each result set is conceptually a table with columns where each
+    column have a name and a type. The name is given as a string and
+    the type is given as a Python type.
+
+    Rows are internally stored as tuples, even if other iterables are
+    used to add a row to the result set.
+
+    :param names: List (or other iterable) of names of columns.
+    :param types: List (or other iterable) of types of columns.
+
+    """
+
+    def __init__(self, names, types):
+        """Constructor.
+
+        See class description for more information.
+
+        """
+
+        # Make sure that we always store the column information as a
+        # tuple, even if other types of iterables are passed to the
+        # function.
+        assert len(names) == len(types)
+        self.__columns = \
+            tuple(ResultSetColumn(nm, tp) for nm, tp in zip(names, types))
+        self.__rows = []
+
+    def table_rows(self):
+        r"""Create rows for the result set as a table.
+
+        This will create the lines for a result set in a tabular
+        format. A typical use-case would be:
+
+        >>> rset = ResultSet(names=('one', 'two'), types=(int, int))
+        >>> rset.append_row([1,2])
+        >>> rset.append_row([3,4])
+        >>> print "\n".join(rset.table_rows())
+        one two
+        --- ---
+          1   2
+          3   4
+
+        """
+        header = [col.name for col in self.__columns]
+        all_rows = [header]
+        all_rows.extend(self.__rows)
+        width = [max(len(str(r)) for r in col) for col in zip(*all_rows)]
+        def _mkline(row):
+            """Create an output string per row.
+            """
+            return " ".join(
+                "{0:>{1}}".format(x, width[i]) for i, x in enumerate(row)
+            )
+
+        result = [_mkline(header)]
+        result.append(_mkline(['-' * w for w in width]))
+        for row in self.__rows:
+            result.append(_mkline(row))
+        return result
+
+    @property
+    def rowcount(self):
+        """The number of rows in the result set
+        """
+        return len(self.__rows)
+
+    @property
+    def columns(self):
+        """An array of the columns defined for the result set.
+        """
+        return self.__columns
+
+    def __str__(self):
+        """Convert the result set to a human-friendly output.
+        """
+        return "\n".join(self.table_rows())
+
+    def __iter__(self):
+        """Iterate over the rows of the result set.
+
+        Each row is a list of column values.
+
+        """
+
+        for row in self.__rows:
+            yield row
+
+    def __getitem__(self, index):
+        """Indexing operator for result set.
+
+        Will index the rows of the result set.
+
+        """
+
+        return self.__rows[index]
+
+    def append_row(self, row):
+        """Append a row to the result set.
+
+        The row is an array (a list or tuple) of values that should be
+        added to the result set. When adding the row, it will be
+        checked that the length of the row matches the number of
+        columns defined for the result set and that the types of the
+        values match the type given for the column.
+
+        :param row: An array of the values to add.
+
+        """
+
+        # Check that the length of the row matches the number of
+        # columns for the result set
+        if len(row) != len(self.__columns):
+            message = "Invalid row length: expected %d, was %d" % (
+                len(self.__columns), len(row)
+            )
+            raise _errors.CommandResultError(message)
+
+        self.__rows.append(
+            tuple(col.type(val) for col, val in zip(self.__columns, row))
+        )
+
+
+class CommandResult(object):
+    """Command result class.
+
+    The command result class contain the result of a procedure
+    execution. This covers any errors returned and zero or more result
+    sets.
+
+    :param error: Error string, or None if there were no error.
+    :param results: List of result sets, or a single result set to add.
+    :param uuid: UUID that identifies the Fabric meta-data.
+    :param ttl: Time-To-Live (TTL) in seconds.
+
+    """
+    TABLE_FORMAT = 'TABLE'
+    JSON_FORMAT = 'JSON'
+
+    OUTPUT_FORMAT = [
+        TABLE_FORMAT, JSON_FORMAT
+    ]
+
+    def __init__(self, error, results=None, uuid=FABRIC_UUID, ttl=None):
+        """Constructor.
+
+        See class description for information.
+        """
+        self.__error = error
+        self.__uuid = uuid
+        self.__ttl = _utils.TTL
+        self.__results = []
+
+        # Check the TTL information if there is any. Otherwise, use
+        # the value specified in the configuration file.
+        if ttl is not None:
+            try:
+                self.__ttl = int(ttl)
+            except ValueError:
+                pass
+
+        # Ensure that results is a list of results even if a single
+        # result (or None) was passed as value.
+        if results is None:
+            results = []
+        elif isinstance(results, ResultSet):
+            results = [results]
+        elif not isinstance(results, collections.Iterable):
+            # The actual contents of the iterable is checked in the
+            # append_result function.
+            raise TypeError("Expected None, ResultSet, or iterable")
+
+        # Append the result sets one by one to perform the standard
+        # checks on adding result sets to the command result.
+        for result in results:
+            self.append_result(result)
+
+    @staticmethod
+    def get_output_format(output_format):
+        """Verify and return output format's type as a string.
+        """
+        try:
+            idx = int(output_format)
+            return CommandResult.OUTPUT_FORMAT[idx]
+        except TypeError:
+            return CommandResult.TABLE_FORMAT
+        except (ValueError, IndexError):
+            pass
+
+        output_format = output_format.upper()
+        if output_format not in CommandResult.OUTPUT_FORMAT:
+            return CommandResult.TABLE_FORMAT
+
+        return output_format
+
+    def emit(self, output, output_format=None):
+        """Write a human-readable version of the command result.
+
+        This will print a human-readable version of the command
+        result, including all result sets, to the output provided.
+
+        :param output: File object to write to.
+        """
+        output_format = CommandResult.get_output_format(output_format)
+
+        if output_format == CommandResult.TABLE_FORMAT:
+            self._table_output(output)
+        elif output_format == CommandResult.JSON_FORMAT:
+            self._json_output(output)
+        output.write("\n")
+
+    def _table_output(self, output):
+        """Produce a table likewise output.
+        """
+        rows = [
+            "Fabric UUID:  %s" % self.uuid,
+            "Time-To-Live: %d" % self.__ttl,
+            "",
+        ]
+
+        if self.__error:
+            rows.append(self.__error)
+        elif self.__results:
+            for rset in self.__results:
+                rows.extend(rset.table_rows())
+                rows.append("")
+
+        output.writelines(row + "\n" for row in rows)
+
+    def _json_output(self, output, *args, **kwargs):
+        """Produce a json output.
+        """
+        out_dict = {}
+
+        out_dict["Fabric UUID"] =  str(self.uuid)
+        out_dict["Time-To-Live"] = self.__ttl
+        out_dict["results"] = None
+        out_dict["error"] = None
+
+        if self.__error:
+            out_dict["error"] = self.__error
+        else:
+            out_dict["results"] = {}
+            for idx, rset in enumerate(self.__results):
+                result = {}
+                rset = self.__results[idx]
+                result["rows"] = [row for row in rset]
+                result["columns"] = [
+                    (col.name, col.type.__name__) for col in rset.columns
+                ]
+                out_dict["results"]["result %s" % idx] = result
+
+        json.dump(out_dict, output, *args, **kwargs)
+
+    def __str__(self):
+        """The command result as a string.
+        """
+        output = StringIO()
+        self.emit(output)
+        return output.getvalue()
+
+    @property
+    def uuid(self):
+        """Fabric node UUID for command result.
+        """
+        return self.__uuid
+
+    @property
+    def ttl(self):
+        """Time-To-Live for command result.
+        """
+        return self.__ttl
+
+    @property
+    def error(self):
+        """Command result error, or None if there were no error.
+        """
+        return self.__error
+
+    @property
+    def results(self):
+        """List of result sets.
+        """
+        return self.__results
+
+    def append_result(self, result):
+        """Append a result set to the list of result sets.
+
+        :param result: Result set to add last to the list of result sets.
+        """
+        if not isinstance(result, ResultSet):
+            raise _errors.CommandResultError(
+                "Result have to be an instance of ResultSet"
+            )
+        if self.error:
+            raise _errors.CommandResultError(
+                "Result sets cannot be added for error results"
+            )
+        self.__results.append(result)
+
+if __name__ == '__main__':
+    import doctest
+    doctest.testmod()
