@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2013,2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2013,2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -53,13 +53,9 @@ import uuid as _uuid
 
 import mysql.fabric.errors as _errors
 
-from mysql.fabric.server_utils import (
+from mysql.fabric.connection import (
     MYSQL_DEFAULT_PORT,
-    connect_to_mysql,
-    exec_mysql_stmt,
-    destroy_mysql_connection,
-    is_valid_mysql_connection,
-    reestablish_mysql_connection
+    MySQLConnection, 
 )
 
 DEFAULT_DATABASE = 'fabric'
@@ -340,21 +336,6 @@ class MySQLPersister(object):
     the object database, and give each subclass of :class:`Persistable` a chance
     to set itself up by calling the class `init` method.
     """
-
-    STORE_PRIVILEGES = [      # GRANT ... ON <fabric_db>.*
-        "ALTER",              # alter some database objects
-        "CREATE",             # create most database objects
-        "CREATE VIEW",        # create view
-        "DELETE",             # delete rows
-        "DROP",               # drop most database objects
-        "EVENT",              # manage events
-        "INDEX",              # create index
-        "INSERT",             # insert rows
-        "REFERENCES",         # foreign keys
-        "SELECT",             # select rows
-        "UPDATE",             # update rows
-    ]
-
     @classmethod
     def init(cls, host, user, password=None, port=None, database=None,
              connection_timeout=None, connection_attempts=None,
@@ -416,11 +397,11 @@ class MySQLPersister(object):
         the database if it does not exist.
         """
         assert (cls.connection_info is not None)
-        conn = connect_to_mysql(
+        conn = MySQLConnection(
             autocommit=True, **cls.connection_info
         )
-        exec_mysql_stmt(
-            conn, "CREATE DATABASE %s DEFAULT CHARSET=utf8" % (cls.database, )
+        conn.exec_stmt(
+            "CREATE DATABASE %s DEFAULT CHARSET=utf8" % (cls.database, )
         )
 
     @classmethod
@@ -432,11 +413,11 @@ class MySQLPersister(object):
         all object tables.
         """
         assert (cls.connection_info is not None)
-        conn = connect_to_mysql(
+        conn = MySQLConnection(
             autocommit=True, **cls.connection_info
         )
-        exec_mysql_stmt(
-            conn, "DROP DATABASE IF EXISTS %s" % (cls.database, )
+        conn.exec_stmt(
+            "DROP DATABASE IF EXISTS %s" % (cls.database, )
         )
 
     def __init__(self):
@@ -447,7 +428,7 @@ class MySQLPersister(object):
 
         assert (self.connection_info is not None)
         try:
-            self.__cnx = connect_to_mysql(
+            self.__cnx = MySQLConnection(
                 autocommit=True, database=self.database,
                 **self.connection_info
             )
@@ -465,7 +446,7 @@ class MySQLPersister(object):
         """
         try:
             if self.__cnx:
-                destroy_mysql_connection(self.__cnx)
+                self.__cnx.shutdown()
         except AttributeError:
             pass
 
@@ -501,9 +482,7 @@ class MySQLPersister(object):
         Otherwise, return None.
         """
         try:
-            row = exec_mysql_stmt(
-                self.__cnx, "SELECT @@GLOBAL.SERVER_UUID"
-            )
+            row = self.__cnx.exec_stmt("SELECT @@GLOBAL.SERVER_UUID")
             return _uuid.UUID(str(row[0][0]))
         except _errors.DatabaseError:
             pass
@@ -513,9 +492,7 @@ class MySQLPersister(object):
     def max_allowed_connections(self):
         """Return the maximum number of allowed connections to server.
         """
-        row = exec_mysql_stmt(
-            self.__cnx, "SELECT @@GLOBAL.max_connections"
-        )
+        row = self.__cnx.exec_stmt("SELECT @@GLOBAL.max_connections")
         return int(row[0][0])
 
     def exec_stmt(self, stmt_str, options=None):
@@ -525,15 +502,13 @@ class MySQLPersister(object):
         the connection is valid or not. If the connection is invalid, it tries
         to restablish it as MySQL might disconnect inactive connections.
 
-        See :meth:`~mysql.fabric.server_utils.exec_stmt`.
+        See :meth:`~mysql.fabric.connection.MySQLConnection.exec_stmt`.
         """
         while True:
             if self.__check_connection and \
-                not is_valid_mysql_connection(self.__cnx):
+                not self.__cnx.is_valid_mysql_connection():
                 self._try_to_fix_connection()
-            return exec_mysql_stmt(
-                self.__cnx, stmt_str, options
-            )
+            return self.__cnx.exec_stmt(stmt_str, options)
 
     def _try_to_fix_connection(self):
         """Try to get a new connection if the current one is stale.
@@ -541,11 +516,11 @@ class MySQLPersister(object):
         for attempt in range(0, self.connection_attempts):
             try:
                 if self.__cnx:
-                    reestablish_mysql_connection(
-                        self.__cnx, attempt=1, delay=0
+                    self.__cnx.reestablish_mysql_connection(
+                        attempt=1, delay=0
                     )
                 else:
-                    self.__cnx = connect_to_mysql(
+                    self.__cnx = MySQLConnection(
                         autocommit=True, database=self.database,
                         **self.connection_info
                     )
@@ -651,9 +626,3 @@ def teardown():
     """
     _LOGGER.info("Teardown persister.")
     MySQLPersister.teardown()
-
-def required_privileges():
-    """Return the list of MySQL server privileges needed for the
-    backing store.
-    """
-    return MySQLPersister.STORE_PRIVILEGES
