@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2014,2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2014 Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,7 +34,6 @@ from mysql.fabric import (
     replication as _replication,
     backup as _backup,
     utils as _utils,
-    config as _config,
 )
 
 from mysql.fabric.server import (
@@ -49,6 +48,8 @@ from mysql.fabric.sharding import (
     Shards,
     SHARDING_DATATYPE_HANDLER,
     SHARDING_SPECIFICATION_HANDLER,
+    SHARD_METADATA,
+    SHARD_METADATA_VERIFIER,
 )
 
 from mysql.fabric.command import (
@@ -122,10 +123,22 @@ class MoveShardServer(ProcedureShard):
         :param synchronous: Whether one should wait until the execution finishes
                         or not.
         """
+        mysqldump_binary = _services_utils.read_config_value(
+                                self.config,
+                                'sharding',
+                                'mysqldump_program'
+                            )
+        mysqlclient_binary = _services_utils.read_config_value(
+                                self.config,
+                                'sharding',
+                                'mysqlclient_program'
+                            )
+        config_file = self.config.config_file if self.config.config_file else ""
 
         procedures = _events.trigger(
             CHECK_SHARD_INFORMATION, self.get_lockable_objects(), shard_id,
-            group_id, None, "", "MOVE", update_only
+            group_id, mysqldump_binary, mysqlclient_binary, None, config_file,
+            "", "MOVE", update_only
         )
         return self.wait_for_procedures(procedures, synchronous)
 
@@ -154,15 +167,27 @@ class SplitShardServer(ProcedureShard):
         :param synchronous: Whether one should wait until the execution
                             finishes
         """
+        mysqldump_binary = _services_utils.read_config_value(
+                                self.config,
+                                'sharding',
+                                'mysqldump_program'
+                            )
+        mysqlclient_binary = _services_utils.read_config_value(
+                                self.config,
+                                'sharding',
+                                'mysqlclient_program'
+                            )
         prune_limit =   _services_utils.read_config_value(
                                 self.config,
                                 'sharding',
                                 'prune_limit'
                             )
+        config_file = self.config.config_file if self.config.config_file else ""
+
         procedures = _events.trigger(
-            CHECK_SHARD_INFORMATION, self.get_lockable_objects(), shard_id,
-            group_id, split_value, prune_limit, "SPLIT", update_only
-        )
+            CHECK_SHARD_INFORMATION, self.get_lockable_objects(),
+            shard_id, group_id, mysqldump_binary, mysqlclient_binary,
+            split_value, config_file, prune_limit, "SPLIT", update_only)
         return self.wait_for_procedures(procedures, synchronous)
 
 
@@ -194,50 +219,24 @@ def _prune_shard_tables(table_name, prune_limit):
             raise error
 
 @_events.on_event(CHECK_SHARD_INFORMATION)
-def _check_shard_information(shard_id, destn_group_id,
-                             split_value, prune_limit, cmd, update_only):
+def _check_shard_information(shard_id, destn_group_id, mysqldump_binary,
+                             mysqlclient_binary, split_value, config_file, prune_limit, cmd,
+                             update_only):
     """Verify the sharding information before starting a re-sharding operation.
 
     :param shard_id: The destination shard ID.
     :param destn_group_id: The Destination group ID.
+    :param mysqldump_binary: The path to the mysqldump binary.
+    :param mysqlclient_binary: The path to the mysqlclient binary.
     :param split_value: The point at which the sharding definition
                         should be split.
+    :param config_file: The complete path to the fabric configuration
+                        file.
     :param prune_limit: The number of DELETEs that should be
                         done in one batch.
     :param cmd: Indicates if it is a split or a move being executed.
     :param update_only: If the operation is a update only operation.
     """
-    backup_user = _services_utils.read_config_value(
-                            _config.global_config,
-                            'servers',
-                            'backup_user'
-                        )
-    backup_passwd = _services_utils.read_config_value(
-                            _config.global_config,
-                            'servers',
-                            'backup_password'
-                        )
-    restore_user = _services_utils.read_config_value(
-                            _config.global_config,
-                            'servers',
-                            'restore_user'
-                        )
-    restore_passwd = _services_utils.read_config_value(
-                            _config.global_config,
-                            'servers',
-                            'restore_password'
-                        )
-    mysqldump_binary = _services_utils.read_config_value(
-                            _config.global_config,
-                            'sharding',
-                            'mysqldump_program'
-                        )
-    mysqlclient_binary = _services_utils.read_config_value(
-                            _config.global_config,
-                            'sharding',
-                            'mysqlclient_program'
-                        )
-
     if not _services_utils.is_valid_binary(mysqldump_binary):
         raise _errors.ShardingError(
                 _services_sharding.MYSQLDUMP_NOT_FOUND % mysqldump_binary)
@@ -320,23 +319,10 @@ def _check_shard_information(shard_id, destn_group_id,
             (destn_group_id, ))
 
     if not update_only:
-        # Check if the source server has backup privileges.
-        source_group = Group.fetch(source_group_id)
-        server = _services_utils.fetch_backup_server(source_group)
-        server.user = backup_user
-        server.passwd = backup_passwd
-        _backup.MySQLDump.check_backup_privileges(server)
-
-        # Check if the destination server has restore privileges.
-        destination_group = Group.fetch(destn_group_id)
-        server = MySQLServer.fetch(destination_group.master)
-        server.user = restore_user
-        server.passwd = restore_passwd
-        _backup.MySQLDump.check_restore_privileges(server)
-
         _events.trigger_within_procedure(
             BACKUP_SOURCE_SHARD, shard_id, source_group_id, destn_group_id,
-            split_value, prune_limit, cmd, update_only
+            mysqldump_binary, mysqlclient_binary, split_value, config_file,
+            prune_limit, cmd, update_only
         )
     else:
         _events.trigger_within_procedure(
@@ -346,44 +332,32 @@ def _check_shard_information(shard_id, destn_group_id,
 
 @_events.on_event(BACKUP_SOURCE_SHARD)
 def _backup_source_shard(shard_id, source_group_id, destn_group_id,
-                         split_value, prune_limit, cmd, update_only):
+                         mysqldump_binary, mysqlclient_binary, split_value,
+                         config_file, prune_limit, cmd, update_only):
     """Backup the source shard.
 
     :param shard_id: The shard ID of the shard that needs to be moved.
     :param source_group_id: The group_id of the source shard.
     :param destn_group_id: The ID of the group to which the shard needs to
                            be moved.
+    :param mysqldump_binary: The fully qualified mysqldump binary.
+    :param mysqlclient_binary: The fully qualified mysql client binary.
     :param split_value: Indicates the value at which the range for the
                         particular shard will be split. Will be set only
                         for shard split operations.
+    :param config_file: The complete path to the fabric configuration file.
     :param prune_limit: The number of DELETEs that should be
                         done in one batch.
     :param cmd: Indicates the type of re-sharding operation (move, split)
     :update_only: Only update the state store and skip provisioning.
     """
-    backup_user = _services_utils.read_config_value(
-                            _config.global_config,
-                            'servers',
-                            'backup_user'
-                        )
-    backup_passwd = _services_utils.read_config_value(
-                            _config.global_config,
-                            'servers',
-                            'backup_password'
-                        )
-    mysqldump_binary = _services_utils.read_config_value(
-                            _config.global_config,
-                            'sharding',
-                            'mysqldump_program'
-                        )
-
     source_group = Group.fetch(source_group_id)
     move_source_server = _services_utils.fetch_backup_server(source_group)
 
     #Do the backup of the group hosting the source shard.
     backup_image = _backup.MySQLDump.backup(
                         move_source_server,
-                        backup_user, backup_passwd,
+                        config_file,
                         mysqldump_binary
                     )
 
@@ -394,46 +368,35 @@ def _backup_source_shard(shard_id, source_group_id, destn_group_id,
                                      shard_id,
                                      source_group_id,
                                      destn_group_id,
+                                     mysqlclient_binary,
                                      backup_image.path,
                                      split_value,
+                                     config_file,
                                      prune_limit,
                                      cmd
                                      )
 
 @_events.on_event(RESTORE_SHARD_BACKUP)
 def _restore_shard_backup(shard_id, source_group_id, destn_group_id,
-                          backup_image, split_value, prune_limit, cmd):
+                            mysqlclient_binary, backup_image,
+                            split_value, config_file, prune_limit, cmd):
     """Restore the backup on the destination Group.
 
     :param shard_id: The shard ID of the shard that needs to be moved.
     :param source_group_id: The group_id of the source shard.
     :param destn_group_id: The ID of the group to which the shard needs to
                            be moved.
+    :param mysqlclient_binary: The fully qualified mysqlclient binary.
     :param backup_image: The destination file that contains the backup
                          of the source shard.
     :param split_value: Indicates the value at which the range for the
                         particular shard will be split. Will be set only
                         for shard split operations.
+    :param config_file: The complete path to the fabric configuration file.
     :param prune_limit: The number of DELETEs that should be
                         done in one batch.
     :param cmd: Indicates the type of re-sharding operation
     """
-    restore_user = _services_utils.read_config_value(
-                            _config.global_config,
-                            'servers',
-                            'restore_user'
-                        )
-    restore_passwd = _services_utils.read_config_value(
-                            _config.global_config,
-                            'servers',
-                            'restore_password'
-                        )
-    mysqlclient_binary = _services_utils.read_config_value(
-                            _config.global_config,
-                            'sharding',
-                            'mysqlclient_program'
-                        )
-
     destn_group = Group.fetch(destn_group_id)
     if destn_group is None:
         raise _errors.ShardingError(_services_sharding.SHARD_GROUP_NOT_FOUND %
@@ -446,8 +409,8 @@ def _restore_shard_backup(shard_id, source_group_id, destn_group_id,
         destn_group_server.connect()
         _backup.MySQLDump.restore_fabric_server(
             destn_group_server,
-            restore_user, restore_passwd,
             bk_img,
+            config_file,
             mysqlclient_binary
         )
 
@@ -627,6 +590,13 @@ def _setup_shard_switch_split(shard_id, source_group_id, destination_group_id,
     range_sharding_spec, source_shard, shard_mappings, shard_mapping_defn = \
             _services_sharding.verify_and_fetch_shard(shard_id)
 
+    if not update_only:
+        #Fetch the metdata from the source shard
+        shard_meta_data = SHARD_METADATA.fetch_shard_meta_data(
+                                        shard_id, source_group_id)
+        lower_bound = shard_meta_data["lower_bound"]
+        upper_bound = shard_meta_data["upper_bound"]
+
     #Disable the old shard
     source_shard.disable()
 
@@ -653,6 +623,13 @@ def _setup_shard_switch_split(shard_id, source_group_id, destination_group_id,
     new_shard_1 = Shards.add(source_shard.group_id, "DISABLED")
     new_shard_2 = Shards.add(destination_group_id, "DISABLED")
 
+    if not update_only:
+        #The backup has been restored on both the new shards. Hence both
+        #of them will have the old trigger ranges defined, which needs to be
+        #changed.
+        SHARD_METADATA.delete_shard_meta_data(source_shard.group_id, shard_id)
+        SHARD_METADATA.delete_shard_meta_data(destination_group_id, shard_id)
+
     #Both of the shard mappings associated with this shard_id should
     #be of the same sharding type. Hence it is safe to use one of the
     #shard mappings.
@@ -716,6 +693,22 @@ def _setup_shard_switch_split(shard_id, source_group_id, destination_group_id,
     _group_replication.setup_group_replication \
             (shard_mapping_defn[2], destination_group_id)
 
+    if not update_only:
+        #update the sharding metadata for both the shards.
+        SHARD_METADATA.insert_shard_meta_data(
+            new_shard_1.shard_id,
+            lower_bound,
+            split_value,
+            new_shard_1.group_id
+        )
+        SHARD_METADATA.insert_shard_meta_data(
+            new_shard_2.shard_id,
+            split_value,
+            upper_bound,
+            new_shard_2.group_id
+        )
+
+
     #Enable the split shards
     new_shard_1.enable()
     new_shard_2.enable()
