@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,16 +18,12 @@
 """mysql.utilities"""
 
 # Major, Minor, Patch, Status
-VERSION = (1, 3, 6, 'GA', 0)
+VERSION = (1, 6, 1, 'Alpha', 0)
 # Future versions will have to include only the X, Y (no Z).
-WORKBENCH_VERSION = (5, 2, 47)
 
 VERSION_STRING = "%s.%s.%s" % VERSION[0:3]
-RELEASE_STRING = (
-    VERSION_STRING +
-    " (part of MySQL Workbench Distribution %s.%s.%s)" % WORKBENCH_VERSION)
 
-COPYRIGHT = "2010, 2013 Oracle and/or its affiliates. All rights reserved."
+COPYRIGHT = "2010, 2015 Oracle and/or its affiliates. All rights reserved."
 
 COPYRIGHT_FULL = "Copyright (c) " + COPYRIGHT + """
 This is a release of dual licensed MySQL Utilities. For the avoidance of
@@ -38,14 +34,15 @@ MySQL Utilities is brought to you by Oracle.
 
 LICENSE = "GPLv2"
 
-VERSION_FRM = ("MySQL Utilities {program} version {RELEASE_STRING} \n"
+VERSION_FRM = ("MySQL Utilities {program} version {VERSION_STRING} \n"
                "License type: {LICENSE}".format(program="{program}",
-                                                RELEASE_STRING=RELEASE_STRING,
+                                                VERSION_STRING=VERSION_STRING,
                                                 LICENSE=LICENSE))
 
 LICENSE_FRM = (VERSION_FRM + "\n" + COPYRIGHT_FULL)
 PYTHON_MIN_VERSION = (2, 6, 0)
 PYTHON_MAX_VERSION = (3, 0, 0)
+CONNECTOR_MIN_VERSION = (1, 2, 1)
 
 # This dictionary has to be updated whenever a utility is added.
 # the format to use is:
@@ -53,6 +50,9 @@ PYTHON_MAX_VERSION = (3, 0, 0)
 AVAILABLE_UTILITIES = {
     'mysqlauditadmin': (),
     'mysqlauditgrep': (),
+    'mysqlbinlogmove': (),
+    'mysqlbinlogpurge': (),
+    'mysqlbinlogrotate': (),
     'mysqldbcompare': (),
     'mysqldbcopy': (),
     'mysqldbexport': (),
@@ -61,15 +61,19 @@ AVAILABLE_UTILITIES = {
     'mysqldiskusage': (),
     'mysqlfailover': (),
     'mysqlfrm': (),
+    'mysqlgrants': (),
     'mysqlindexcheck': (),
     'mysqlmetagrep': (),
     'mysqlprocgrep': (),
     'mysqlreplicate': (),
     'mysqlrpladmin': (),
     'mysqlrplcheck': (),
+    'mysqlrplms': (),
     'mysqlrplshow': (),
+    'mysqlrplsync': (),
     'mysqlserverclone': (),
     'mysqlserverinfo': (),
+    'mysqlslavetrx': (),
     'mysqluc': (),
     'mysqluserclone': (),
 }
