@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ from mysql.utilities.common.dbcompare import (diff_objects, get_common_objects,
                                               check_consistency,
                                               build_diff_list,
                                               DEFAULT_SPAN_KEY_SIZE)
-
+from mysql.utilities.common.server import connect_servers
 
 _PRINT_WIDTH = 75
 _ROW_FORMAT = "# {0:{1}} {2:{3}} {4:{5}} {6:{7}} {8:{9}}"
@@ -39,6 +39,7 @@ _ERROR_DB_DIFF = "The object definitions do not match."
 _ERROR_DB_MISSING = "The database {0} does not exist."
 _ERROR_OBJECT_LIST = "The list of objects differs among database {0} and {1}."
 _ERROR_ROW_COUNT = "Row counts are not the same among {0} and {1}.\n#"
+_ERROR_DB_MISSING_ON_SERVER = "The database {0} on {1} does not exist on {2}."
 
 _DEFAULT_OPTIONS = {
     "quiet": False,
@@ -297,25 +298,21 @@ def _check_data_consistency(server1, server2, obj1, obj2, reporter, options):
     reporter[in]      database compare reporter class instance
     options[in]       options dictionary
 
-    Returns list of errors
+    Returns list of errors debug_msgs
     """
     direction = options.get('changes-for', 'server1')
     reverse = options.get('reverse', False)
 
     errors = []
-    diff_server1 = []
-    diff_server2 = []
-    diff_list = []
+    debug_msgs = []
     # For each table, do row data consistency check
     if not options['no_data']:
+        reporter.report_state('-')
         try:
-            # Do the comparison based on direction
-            if direction == 'server1' or reverse:
-                diff_server1 = check_consistency(server1, server2,
-                                                 obj1, obj2, options)
-            if direction == 'server2' or reverse:
-                diff_server2 = check_consistency(server2, server1,
-                                                 obj2, obj1, options)
+            # Do the comparison considering the direction.
+            diff_server1, diff_server2 = check_consistency(
+                server1, server2, obj1, obj2, options, diag_msgs=debug_msgs,
+                reporter=reporter)
 
             # if no differences, return
             if (diff_server1 is None and diff_server2 is None) or \
@@ -323,8 +320,7 @@ def _check_data_consistency(server1, server2, obj1, obj2, reporter, options):
                      diff_server1 is None) or \
                     (not reverse and direction == 'server2' and
                      diff_server2 is None):
-                reporter.report_state('pass')
-                return errors
+                return errors, debug_msgs
 
             # Build diff list
             new_opts = options.copy()
@@ -337,13 +333,10 @@ def _check_data_consistency(server1, server2, obj1, obj2, reporter, options):
                 diff_list = build_diff_list(diff_server2, diff_server1,
                                             diff_server2, diff_server1,
                                             'server2', 'server1', new_opts)
-            if len(diff_list) == 0:
-                reporter.report_state('pass')
-            else:
-                reporter.report_state('FAIL')
+            if diff_list:
                 errors = diff_list
         except UtilError, e:
-            if e.errmsg == "No primary key found.":
+            if e.errmsg.endswith("not have an usable Index or primary key."):
                 reporter.report_state('SKIP')
                 errors.append("# {0}".format(e.errmsg))
             else:
@@ -353,10 +346,11 @@ def _check_data_consistency(server1, server2, obj1, obj2, reporter, options):
                     raise e
                 else:
                     errors.append(e.errmsg)
+
     else:
         reporter.report_state('SKIP')
 
-    return errors
+    return errors, debug_msgs
 
 
 def _check_option_defaults(options):
@@ -447,12 +441,10 @@ def database_compare(server1_val, server2_val, db1, db2, options):
     # Remaining operations can occur in a loop one for each object.
     for item in in_both:
         error_list = []
-
+        debug_msgs = []
         # Set the object type
         obj_type = item[0]
 
-        obj1 = "{0}.{1}".format(db1, item[1][0])
-        obj2 = "{0}.{1}".format(db2, item[1][0])
         q_obj1 = "{0}.{1}".format(quote_with_backticks(db1),
                                   quote_with_backticks(item[1][0]))
         q_obj2 = "{0}.{1}".format(quote_with_backticks(db2),
@@ -476,8 +468,9 @@ def database_compare(server1_val, server2_val, db1, db2, options):
 
         # Check data consistency for tables
         if obj_type == 'TABLE':
-            errors = _check_data_consistency(server1, server2, q_obj1, q_obj2,
-                                             reporter, options)
+            errors, debug_msgs = _check_data_consistency(server1, server2,
+                                                         q_obj1, q_obj2,
+                                                         reporter, options)
             if len(errors) != 0:
                 error_list.extend(errors)
         else:
@@ -485,8 +478,11 @@ def database_compare(server1_val, server2_val, db1, db2, options):
 
         if options['verbosity'] > 0:
             print
-            get_create_object(server1, obj1, options, obj_type)
-            get_create_object(server2, obj2, options, obj_type)
+            get_create_object(server1, q_obj1, options, obj_type)
+            get_create_object(server2, q_obj2, options, obj_type)
+
+        if debug_msgs and options['verbosity'] > 2:
+            reporter.report_errors(debug_msgs)
 
         reporter.report_errors(error_list)
 
@@ -495,3 +491,89 @@ def database_compare(server1_val, server2_val, db1, db2, options):
             success = False
 
     return success
+
+
+def compare_all_databases(server1_val, server2_val, exclude_list, options):
+    """Perform a consistency check among all common databases on the servers
+
+    This method gets all databases from the servers, prints any missing
+    databases and performs a consistency check among all common databases.
+
+    If any errors or differences are found, the operation will print the
+    difference and continue.
+
+    This method will return None if no databases to compare.
+    """
+
+    success = True
+
+    # Connect to servers
+    conn_options = {
+        "quiet": options.get("quiet", False),
+        "src_name": "server1",
+        "dest_name": "server2",
+
+    }
+    server1, server2 = connect_servers(server1_val, server2_val, conn_options)
+
+    # Check if the specified servers are the same.
+    if server1.port == server2.port and server1.is_alias(server2.host):
+        raise UtilError(
+            "Specified servers are the same (server1={host1}:{port1} and "
+            "server2={host2}:{port2}). Cannot compare all databases on the "
+            "same server.".format(host1=server1.host, port1=server1.port,
+                                  host2=server2.host, port2=server2.port))
+
+    # Get all databases, except those used in --exclude
+    get_dbs_query = """
+        SELECT SCHEMA_NAME
+        FROM INFORMATION_SCHEMA.SCHEMATA
+        WHERE SCHEMA_NAME != 'INFORMATION_SCHEMA'
+        AND SCHEMA_NAME != 'PERFORMANCE_SCHEMA'
+        AND SCHEMA_NAME != 'mysql'
+        {0}"""
+    conditions = ""
+    if exclude_list:
+        # Add extra where to exclude databases in exclude_list
+        operator = 'REGEXP' if options['use_regexp'] else 'LIKE'
+        conditions = "AND {0}".format(" AND ".join(
+            ["SCHEMA_NAME NOT {0} '{1}'".format(operator, db)
+             for db in exclude_list])
+        )
+
+    server1_dbs = set(
+        [db[0] for db in server1.exec_query(get_dbs_query.format(conditions))]
+    )
+    server2_dbs = set(
+        [db[0] for db in server2.exec_query(get_dbs_query.format(conditions))]
+    )
+
+    # Check missing databases
+    if options['changes-for'] == 'server1':
+        diff_dbs = server1_dbs.difference(server2_dbs)
+        for db in diff_dbs:
+            msg = _ERROR_DB_MISSING_ON_SERVER.format(db, "server1", "server2")
+            print("# {0}".format(msg))
+    else:
+        diff_dbs = server2_dbs.difference(server1_dbs)
+        for db in diff_dbs:
+            msg = _ERROR_DB_MISSING_ON_SERVER.format(db, "server2", "server1")
+            print("# {0}".format(msg))
+
+    # Compare databases in common
+    common_dbs = server1_dbs.intersection(server2_dbs)
+    if common_dbs:
+        print("# Comparing databases: {0}".format(", ".join(common_dbs)))
+    else:
+        success = None
+    for db in common_dbs:
+        try:
+            res = database_compare(server1_val, server2_val, db, db, options)
+            if not res:
+                success = False
+            print("\n")
+        except UtilError as err:
+            print("ERROR: {0}\n".format(err.errmsg))
+            success = False
+
+    return success
