@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@ from mysql.utilities.exception import UtilError
 from mysql.utilities.common.server import connect_servers
 from mysql.utilities.common.database import Database
 from mysql.utilities.common.table import Table
+from mysql.utilities.common.sql_transform import quote_with_backticks
 
 
 def check_index(src_val, table_args, options):
@@ -47,6 +48,7 @@ def check_index(src_val, table_args, options):
                          index-format : index format = sql, table, tab, csv
                          worst        : show worst performing indexes
                          best         : show best performing indexes
+                         report-indexes : reports tables without PK or UK
 
     Returns bool True = success, raises UtilError if error
     """
@@ -60,6 +62,7 @@ def check_index(src_val, table_args, options):
     stats = options.get("stats", False)
     first_indexes = options.get("best", None)
     last_indexes = options.get("worst", None)
+    report_indexes = options.get("report-indexes", False)
 
     # Try to connect to the MySQL database server.
     conn_options = {
@@ -97,7 +100,8 @@ def check_index(src_val, table_args, options):
         if not tables and verbosity >= 1:
             print "# Warning: database %s does not exist. Skipping." % (db)
         for table in tables:
-            table_list.append(db + "." + table[0])
+            table_list.append("{0}.{1}".format(quote_with_backticks(db),
+                                               quote_with_backticks(table[0])))
 
     # Fail if no tables to check
     if not table_list:
@@ -119,15 +123,20 @@ def check_index(src_val, table_args, options):
                             "to skip missing tables." % table_name)
         if exists:
             if not tbl.get_indexes():
-                if verbosity > 1:
+                if verbosity > 1 or report_indexes:
                     print "# Table %s is not indexed." % (table_name)
             else:
                 if show_indexes:
-                    tbl.print_indexes(index_format)
+                    tbl.print_indexes(index_format, verbosity)
                     # Show if table has primary key
-                if not tbl.has_primary_key():
-                    if verbosity > 1:
-                        print "#   Table %s does not contain a PRIMARY key."
+                if verbosity > 1 or report_indexes:
+                    if not tbl.has_primary_key():
+                        if not tbl.has_unique_key():
+                            print("# Table {0} does not contain neither a "
+                                  "PRIMARY nor UNIQUE key.".format(table_name))
+                        else:
+                            print("# Table {0} does not contain a PRIMARY key."
+                                  "".format(table_name))
                 tbl.check_indexes(show_drops)
 
             # Show best and/or worst indexes
