@@ -20,6 +20,7 @@ This file contains the reporting mechanisms for reporting disk usage.
 """
 import getpass
 import os
+import shlex
 import subprocess
 import sys
 import tempfile
@@ -181,15 +182,19 @@ def _server_info(server_val, get_defaults=False, options=None):
                 log_file = params_dict[log_tpl.log_name]
 
             # Now get the information about the size of the logs
-            try:
-                params_dict[log_tpl.log_file_size] = "{0} bytes".format(
-                    os.path.getsize(log_file))
+            # If log file is stderr, we cannot get the correct size.
+            if log_file not in ["stderr", "stdout"]:
+                # Now get the information about the size of the logs
+                try:
+                    params_dict[log_tpl.log_file_size] = "{0} bytes".format(
+                        os.path.getsize(log_file))
+
+                except os.error:
+                    # if we are unable to get the log_file_size
+                    params_dict[log_tpl.log_file_size] = ''
+                    warning_msg = _WARNING_TEMPLATE.format(msg, log_file)
+                    params_dict['warnings'].append(warning_msg)
 
-            except os.error:
-                # if we are unable to get the log_file_size
-                params_dict[log_tpl.log_file_size] = ''
-                warning_msg = _WARNING_TEMPLATE.format(msg, log_file)
-                params_dict['warnings'].append(warning_msg)
         else:
             params_dict['warnings'].append("Unable to get information "
                                            "regarding variable '{0}'"
@@ -243,7 +248,7 @@ def _server_info(server_val, get_defaults=False, options=None):
         if server.is_alias('localhost'):
             try:
                 my_def_path = get_tool_path(params_dict['basedir'],
-                                            "my_print_defaults")
+                                            "my_print_defaults", quote=True)
             except UtilError as err:
                 raise UtilError("Unable to retrieve the defaults data "
                                 "(requires access to my_print_defaults): {0} "
@@ -252,7 +257,9 @@ def _server_info(server_val, get_defaults=False, options=None):
                                 )
             out_file = tempfile.TemporaryFile()
             # Execute tool: <basedir>/my_print_defaults mysqld
-            subprocess.call([my_def_path, "mysqld"], stdout=out_file)
+            cmd_list = shlex.split(my_def_path)
+            cmd_list.append("mysqld")
+            subprocess.call(cmd_list, stdout=out_file)
             out_file.seek(0)
             # Get defaults data from temp output file.
             defaults.append("\nDefaults for server {0}".format(server_id))
@@ -302,7 +309,7 @@ def _start_server(server_val, basedir, datadir, options=None):
     verbosity = options.get("verbosity", 0)
     start_timeout = options.get("start_timeout", 10)
 
-    mysqld_path = get_tool_path(basedir, "mysqld")
+    mysqld_path = get_tool_path(basedir, "mysqld", quote=True)
 
     print "# Server is offline."
 
@@ -310,8 +317,15 @@ def _start_server(server_val, basedir, datadir, options=None):
     print "# Checking server version ...",
     version = get_mysqld_version(mysqld_path)
     print "done."
-    post_5_6 = version is not None and \
-        int(version[0]) >= 5 and int(version[1]) >= 6
+    if version is not None and int(version[0]) >= 5:
+        post_5_5 = int(version[1]) >= 5
+        post_5_6 = int(version[1]) >= 6
+        post_5_7_4 = int(version[1]) >= 7 and int(version[2]) > 4
+    else:
+        print("# Warning: cannot get server version.")
+        post_5_5 = False
+        post_5_6 = False
+        post_5_7_4 = False
 
     # Get the user executing the utility to use in the mysqld options.
     # Note: the option --user=user_name is mandatory to start mysqld as root.
@@ -323,7 +337,8 @@ def _start_server(server_val, basedir, datadir, options=None):
         print "# --- BEGIN (server output) ---"
     else:
         print "# Starting read-only instance of the server ...",
-    args = [
+    args = shlex.split(mysqld_path)
+    args.extend([
         "--no-defaults",
         "--skip-grant-tables",
         "--read_only",
@@ -331,19 +346,20 @@ def _start_server(server_val, basedir, datadir, options=None):
         "--basedir=" + basedir,
         "--datadir=" + datadir,
         "--user={0}".format(user_name),
-    ]
+    ])
 
     # It the server is 5.6 or later, we must use additional parameters
-    if post_5_6:
-        args_5_6 = [
+    if post_5_5:
+        server_args = [
             "--skip-slave-start",
-            "--skip-innodb",
             "--default-storage-engine=MYISAM",
-            "--default-tmp-storage-engine=MYISAM",
             "--server-id=0",
         ]
-        args.extend(args_5_6)
-    args.insert(0, mysqld_path)
+        if post_5_6:
+            server_args.append("--default-tmp-storage-engine=MYISAM")
+        if not post_5_7_4:
+            server_args.append("--skip-innodb")
+        args.extend(server_args)
 
     socket = server_val.get('unix_socket', None)
     if socket is not None:
@@ -409,7 +425,7 @@ def _stop_server(server_val, basedir, options=None):
         options = {}
     verbosity = options.get("verbosity", 0)
     socket = server_val.get("unix_socket", None)
-    mysqladmin_path = get_tool_path(basedir, "mysqladmin")
+    mysqladmin_path = get_tool_path(basedir, "mysqladmin", quote=True)
 
     # Stop the instance
     if verbosity > 0:
@@ -505,15 +521,21 @@ def show_server_info(servers, options):
         else:
             _show_running_servers()
 
+    ssl_dict = {}
+    ssl_dict['ssl_cert'] = options.get("ssl_cert", None)
+    ssl_dict['ssl_ca'] = options.get("ssl_ca", None)
+    ssl_dict['ssl_key'] = options.get("ssl_key", None)
+    ssl_dict['ssl'] = options.get("ssl", None)
+
     row_dict_lst = []
     warnings = []
     server_val = {}
     for server in servers:
         new_server = None
         try:
-            test_connect(server, True)
+            test_connect(server, throw_errors=True, ssl_dict=ssl_dict)
         except UtilError as util_error:
-            conn_dict = get_connection_dictionary(server)
+            conn_dict = get_connection_dictionary(server, ssl_dict=ssl_dict)
             server1 = Server(options={'conn_info': conn_dict})
             server_is_off = False
             # If we got errno 2002 it means can not connect through the
