@@ -83,7 +83,9 @@ def to_sql(obj):
 
     Returns (string) converted value
     """
-    return MySQLConverter().quote(obj)
+    to_sql.__dict__.setdefault('converter', MySQLConverter())
+    obj = to_sql.converter.escape(obj)  # pylint: disable=E1101
+    return str(to_sql.converter.quote(obj))  # pylint: disable=E1101
 
 
 def quote_with_backticks(identifier):
@@ -190,12 +192,15 @@ def build_pkey_where_clause(table, row):
     if len(pkeys) > 0:
         col_names = table.get_col_names()
         where_str += "WHERE "
+        pkey_cond_lst = []
         for pkey in pkeys:
             key_col = pkey[0]                         # get the column name
             col_data = row[col_names.index(key_col)]  # get column value
             # quote key column with backticks
             q_key_col = quote_with_backticks(key_col)
-            where_str += "%s = %s" % (q_key_col, to_sql(col_data))
+            pkey_cond_lst.append("{0} = {1}".format(q_key_col,
+                                                    to_sql(col_data)))
+        where_str = "{0}{1}".format(where_str, ' AND '.join(pkey_cond_lst))
 
     return where_str
 
@@ -209,7 +214,7 @@ def build_set_clauses(table, table_cols, dest_row, src_row):
 
     Returns string - WHERE clause or "" if no keys
     """
-    col_metadata = table.get_column_metadata()
+    table.get_column_metadata()
     # do SETs
     set_str = ""
     do_comma = False
@@ -222,8 +227,7 @@ def build_set_clauses(table, table_cols, dest_row, src_row):
                 set_str = "SET "
                 do_comma = True
             # Check for NULL for non-text fields that have no value in new row
-            if len(src_row[col_idx]) == 0 \
-               and not col_metadata[col_idx]['is_text']:
+            if src_row[col_idx] is None:
                 set_str += "%s = %s" % (table_cols[col_idx], "NULL")
             else:
                 set_str += "%s = %s" % (table_cols[col_idx],
@@ -325,7 +329,7 @@ class SQLTransformer(object):
     """
 
     def __init__(self, destination_db, source_db, destination,
-                 source, obj_type, verbosity):
+                 source, obj_type, verbosity, options=None):
         """Constructor
 
         destination_db[in] destination Database instance
@@ -334,6 +338,7 @@ class SQLTransformer(object):
         source[in]         the source object definition or data
         obj_type[in]       type of object
         verbosity[in]      verbosity level
+        options[in]        Options dictionary
 
         """
         self.destination_db = destination_db
@@ -344,6 +349,9 @@ class SQLTransformer(object):
         self.verbosity = verbosity
         self.dest_tbl = None
         self.src_tbl = None
+        if options is None:
+            options = {}
+        self.skip_table_opts = options.get("skip_table_opts", False)
 
     def transform_definition(self):
         """Transform an object definition
@@ -1123,8 +1131,11 @@ class SQLTransformer(object):
             statements.extend(columns[i])
 
         # General definition returns a single string of the option changes
-        gen_defn = self._get_table_defns(self.destination[_TABLE_DEF],
-                                         self.source[_TABLE_DEF])
+        if not self.skip_table_opts:
+            gen_defn = self._get_table_defns(self.destination[_TABLE_DEF],
+                                             self.source[_TABLE_DEF])
+        else:
+            gen_defn = None
 
         if gen_defn is not None:
             statements.append(gen_defn)
