@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,16 +19,19 @@
 This module contains methods for working with mysql server tools.
 """
 
+import ctypes
 import inspect
 import os
 import re
 import sys
+import shlex
 import shutil
 import socket
 import subprocess
 import time
 
-from mysql.utilities import PYTHON_MIN_VERSION, PYTHON_MAX_VERSION
+from mysql.utilities import (PYTHON_MIN_VERSION, PYTHON_MAX_VERSION,
+                             CONNECTOR_MIN_VERSION)
 from mysql.utilities.common.format import print_list
 from mysql.utilities.exception import UtilError
 
@@ -53,7 +56,7 @@ def _add_basedir(search_paths, path_str):
 
 
 def get_tool_path(basedir, tool, fix_ext=True, required=True,
-                  defaults_paths=None, search_PATH=False):
+                  defaults_paths=None, search_PATH=False, quote=False):
     """Search for a MySQL tool and return the full path
 
     basedir[in]         The initial basedir to search (from mysql server)
@@ -69,12 +72,17 @@ def get_tool_path(basedir, tool, fix_ext=True, required=True,
                         the PATH environment variable will be used to search
                         for the tool. By default the PATH will not be searched,
                         i.e. search_PATH=False.
+    quote[in]           If True, the result path is surrounded with the OS
+                        quotes.
     Returns (string) full path to tool
     """
     if not defaults_paths:
         defaults_paths = []
     search_paths = []
-
+    if quote:
+        quote_char = "'" if os.name == "posix" else '"'
+    else:
+        quote_char = ''
     if basedir:
         # Add specified basedir path to search paths
         _add_basedir(search_paths, basedir)
@@ -101,12 +109,12 @@ def get_tool_path(basedir, tool, fix_ext=True, required=True,
         if os.path.isdir(norm_path):
             toolpath = os.path.join(norm_path, tool)
             if os.path.isfile(toolpath):
-                return toolpath
+                return r"{0}{1}{0}".format(quote_char, toolpath)
             else:
                 if tool == "mysqld.exe":
                     toolpath = os.path.join(norm_path, "mysqld-nt.exe")
                     if os.path.isfile(toolpath):
-                        return toolpath
+                        return r"{0}{1}{0}".format(quote_char, toolpath)
     if required:
         raise UtilError("Cannot find location of %s." % tool)
 
@@ -132,6 +140,36 @@ def delete_directory(path):
             shutil.rmtree(path, True)
 
 
+def estimate_free_space(path, unit_multiple=2):
+    """Estimated free space for the given path.
+
+    Calculates free space for the given path, returning the value
+    on the size given by the unit_multiple.
+
+    path[in]             the path to calculate the free space for.
+    unit_multiple[in]    the unit size given as a multiple.
+                         Accepts int values > to zero.
+                         Size    unit_multiple
+                          bytes        0
+                          Kilobytes    1
+                          Megabytes    2
+                          Gigabytes    3
+                         and so on...
+
+    Returns folder/drive free space (in bytes)
+    """
+    unit_size = 1024 ** unit_multiple
+    if os.name == 'nt':
+        free_bytes = ctypes.c_ulonglong(0)
+        ctypes.windll.kernel32.GetDiskFreeSpaceExW(ctypes.c_wchar_p(path),
+                                                   None, None,
+                                                   ctypes.pointer(free_bytes))
+        return free_bytes.value / unit_size
+    else:
+        st = os.statvfs(path)  # pylint: disable=E1101
+        return st.f_bavail * st.f_frsize / unit_size
+
+
 def execute_script(run_cmd, filename=None, options=None, verbosity=False):
     """Execute a script.
 
@@ -158,15 +196,21 @@ def execute_script(run_cmd, filename=None, options=None, verbosity=False):
             filename = os.devnull
         f_out = open(filename, 'w')
 
-    str_opts = [str(opt) for opt in options]
-    cmd_opts = " ".join(str_opts)
-    command = " ".join([run_cmd, cmd_opts])
+    is_posix = True if os.name == "posix" else False
+    command = shlex.split(run_cmd, posix=is_posix)
+
+    if options:
+        command.extend([str(opt) for opt in options])
 
     if verbosity:
         print("# SCRIPT EXECUTED: {0}".format(command))
 
-    proc = subprocess.Popen(command, shell=True,
-                            stdout=f_out, stderr=f_out)
+    try:
+        proc = subprocess.Popen(command, shell=False,
+                                stdout=f_out, stderr=f_out)
+    except OSError as err:
+        raise UtilError(str(err))
+
     ret_val = proc.wait()
     if not verbosity:
         f_out.close()
@@ -219,7 +263,11 @@ def get_mysqld_version(mysqld_path):
 
     if line is None:
         return None
-    version = line.split(' ', 5)[3]
+    pattern = r"mysqld(?:\.exe)?\s+Ver\s+(\d+\.\d+\.\S+)\s"
+    match = re.search(pattern, line)
+    if not match:
+        return None
+    version = match.group(1)
     try:
         maj_ver, min_ver, dev = version.split(".")
         rel = dev.split("-")
@@ -487,13 +535,23 @@ def decode(orig_str):
     return "".join(new_parts)
 
 
-def check_connector_python(print_error=True):
-    """ Check to see if Connector/Python is installed and accessible
+def check_connector_python(print_error=True,
+                           min_version=CONNECTOR_MIN_VERSION):
 
-    print_error[in]     if True, print the error. Default True
+    """Check to see if Connector Python is installed and accessible and
+    meets minimum required version.
+
+    By default this method uses constants to define the minimum
+    C/Python version required. It's possible to override this by passing  a new
+    value to ``min_version`` parameter.
+
+    print_error[in]               If True, print error else do not print
+                                  error on failure.
+    min_version[in]               Tuple with the minimum C/Python version
+                                  required (inclusive).
 
-    Prints error and returns False on failure to find connector.
     """
+    is_compatible = True
     try:
         import mysql.connector  # pylint: disable=W0612
     except ImportError:
@@ -503,7 +561,22 @@ def check_connector_python(print_error=True):
                   "Please check your paths or download and install the "
                   "Connector/Python from http://dev.mysql.com.")
         return False
-    return True
+    else:
+        try:
+            sys_version = mysql.connector.version.VERSION[:3]
+        except AttributeError:
+            is_compatible = False
+
+    if is_compatible and sys_version >= min_version:
+        return True
+    else:
+        if print_error:
+            print("ERROR: The MYSQL Connector/Python module was found "
+                  "but it is either not properly installed or it is an "
+                  "old version. MySQL Utilities requires Connector/Python "
+                  "version > '{0}'. Download and install Connector/Python "
+                  "from http://dev.mysql.com.".format(min_version))
+        return False
 
 
 def print_elapsed_time(start_time):
@@ -514,3 +587,23 @@ def print_elapsed_time(start_time):
     stop_time = time.time()
     display_time = stop_time - start_time
     print("Time: {0:.2f} sec\n".format(display_time))
+
+
+def join_and_build_str(list_of_strings, sep=', ', last_sep='and'):
+    """Buils and returns a string from a list of elems.
+
+    list_of_strings[in]    the list of strings that will be joined into a
+                           single string.
+    sep[in]                the separator that will be used to group all strings
+                           except the last one.
+    last_sep[in]           the separator that is used in last place
+    """
+    if list_of_strings:
+        if len(list_of_strings) > 1:
+            res_str = "{0} {1} {2}".format(
+                sep.join(list_of_strings[:-1]), last_sep, list_of_strings[-1])
+        else:  # list has a single elem
+            res_str = list_of_strings[0]
+    else:  # if list_of_str is empty, return empty string
+        res_str = ""
+    return res_str
