@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@ import logging
 import time
 import operator
 
+from multiprocessing.pool import ThreadPool
+
 from mysql.utilities.exception import FormatError, UtilError, UtilRplError
 from mysql.utilities.common.lock import Lock
 from mysql.utilities.common.my_print_defaults import MyDefaultsReader
@@ -34,7 +36,8 @@ from mysql.utilities.common.tools import execute_script
 from mysql.utilities.common.format import print_list
 from mysql.utilities.common.user import User
 from mysql.utilities.common.server import (get_server_state, get_server,
-                                           get_connection_dictionary)
+                                           get_connection_dictionary,
+                                           log_server_version)
 
 
 _HEALTH_COLS = ["host", "port", "role", "state", "gtid_mode", "health"]
@@ -57,7 +60,7 @@ _SELECT_RPL_USER_PASS_QUERY = ('SELECT user, host, grant_priv, password, '
                                'WHERE user ="%s" AND host ="%s";')
 
 
-def parse_failover_connections(options):
+def parse_topology_connections(options, parse_candidates=True):
     """Parse the --master, --slaves, and --candidates options
 
     This method returns a tuple with server connection dictionaries for
@@ -113,7 +116,7 @@ def parse_failover_connections(options):
                 raise UtilRplError(msg)
 
     candidates_val = []
-    if options.candidates:
+    if parse_candidates and options.candidates:
         candidates = options.candidates.split(",")
         for slave in candidates:
             try:
@@ -252,15 +255,22 @@ class Topology(Replication):
         master = None
         slaves = []
 
+        # Set verbose value.
+        verbose = self.options.get("verbosity", 0) > 0
+
         # attempt to connect to the master
         if master_vals:
-            master = get_server('master', master_vals, True)
+            master = get_server('master', master_vals, True, verbose=verbose)
+            if self.logging:
+                log_server_version(master)
 
         for slave_val in slave_vals:
             host = slave_val['host']
             port = slave_val['port']
             try:
-                slave = get_server('slave', slave_val, True)
+                slave = get_server('slave', slave_val, True, verbose=verbose)
+                if self.logging:
+                    log_server_version(slave)
             except:
                 msg = "Cannot connect to slave %s:%s as user '%s'." % \
                       (host, port, slave_val['user'])
@@ -385,7 +395,11 @@ class Topology(Replication):
                     conn_dict = {
                         'conn_info': {'user': user, 'passwd': password,
                                       'host': host, 'port': port,
-                                      'socket': None},
+                                      'socket': None,
+                                      'ssl_ca': self.master.ssl_ca,
+                                      'ssl_cert': self.master.ssl_cert,
+                                      'ssl_key': self.master.ssl_key,
+                                      'ssl': self.master.ssl},
                         'role': slave,
                         'verbose': self.options.get("verbosity", 0) > 0,
                     }
@@ -402,6 +416,8 @@ class Topology(Replication):
                             self._report(msg, logging.INFO, False)
                             if output_log:
                                 print("# {0}".format(msg))
+                            if self.logging:
+                                log_server_version(slave_conn)
                             new_slaves_found = True
                         else:
                             msg = ("Slave skipped (IO not running): "
@@ -514,7 +530,7 @@ class Topology(Replication):
             if self.force and candidate_ok[1] == "RPL_USER":
                 user, passwd = slave.get_rpl_user()
                 res = candidate.create_rpl_user(slave.host, slave.port,
-                                                user, passwd)
+                                                user, passwd, self.ssl)
                 if not res[0]:
                     print("# ERROR: {0}".format(res[1]))
                     self._report(res[1], logging.CRITICAL, False)
@@ -1062,7 +1078,7 @@ class Topology(Replication):
                 continue
             tnx_set = slave.get_executed_gtid_set()
 
-            # Get master UUID from salve if master is not available
+            # Get master UUID from slave if master is not available
             if use_master_uuid_from_slave:
                 master_uuid = slave.get_master_uuid()
 
@@ -1078,10 +1094,14 @@ class Topology(Replication):
                         _GTID_SUBTRACT_TO_EXECUTED.format(tnx_set))
 
                     # Only consider the transaction as errant if not from the
-                    # current master
+                    # current master.
+                    # Note: server UUID can appear with mixed cases (e.g. for
+                    # 5.6.9 servers the server_uuid is lower case and appears
+                    # in upper cases in the GTID_EXECUTED set.
                     errant_set = set()
                     for tnx in errant_res:
-                        if tnx[0] and not tnx[0].startswith(master_uuid):
+                        if tnx[0] and not tnx[0].lower().startswith(
+                                master_uuid.lower()):
                             errant_set.update(tnx[0].split(',\n'))
 
                     # Errant transactions exist on only one slave, therefore if
@@ -1152,35 +1172,40 @@ class Topology(Replication):
 
         return True
 
-    def _remove_slave(self, slave):
+    def remove_slave(self, slave):
         """Remove a slave from the slaves dictionary list
 
         slave[in]      the dictionary for the slave to remove
         """
-        i = 0
-        for slave_dict in self.slaves:
-            if slave_dict['host'] == slave['host'] and \
-               int(slave_dict['port']) == int(slave['port']):
+        for i, slave_dict in enumerate(self.slaves):
+            if (slave_dict['instance'] and
+                    slave_dict['instance'].is_alias(slave['host']) and
+                    int(slave_dict['port']) == int(slave['port'])):
                 # Disconnect to satisfy new server restrictions on termination
                 self.slaves[i]['instance'].disconnect()
                 self.slaves.pop(i)
                 break
-            i += 1
 
     def gtid_enabled(self):
         """Check if topology has GTID turned on.
 
-        Returns bool - True = GTID_MODE=ON.
+        This method check if GTID mode is turned ON for all servers in the
+        replication topology, skipping the check for not available servers.
+
+        Returns bool - True = GTID_MODE=ON for all available servers (master
+        and slaves) in the replication topology..
         """
-        if self.master is not None:
-            return self.master.supports_gtid() == "ON"
+        if self.master and self.master.supports_gtid() != "ON":
+            return False  # GTID disabled or not supported.
         for slave_dict in self.slaves:
             slave = slave_dict['instance']
             # skip dead or zombie slaves
             if slave is None or not slave.is_alive():
                 continue
-            return slave.supports_gtid() == "ON"
-        return False
+            if slave.supports_gtid() != "ON":
+                return False  # GTID disabled or not supported.
+        # GTID enabled for all topology (excluding not available servers).
+        return True
 
     def get_servers_with_gtid_not_on(self):
         """Get the list of servers from the topology with GTID turned off.
@@ -1240,48 +1265,59 @@ class Topology(Replication):
               IO_Thread, SQL_Thread, Secs_Behind, Remaining_Delay,
               IO_Error_Num, IO_Error
 
+        Note: The method will return health for the master and slaves or just
+              the slaves if no master is specified. In which case, the master
+              status shall display "no master specified" instead of a status
+              for the connection.
+
         Returns tuple - (columns, rows)
         """
-        assert (self.master is not None), "No master or connection failed."
-
-        # Get master health
-        rpl_health = self.master.check_rpl_health()
-        self._report("# Getting health for master: %s:%s." %
-                     (self.master.host, self.master.port), logging.INFO, False)
-        have_gtid = self.master.supports_gtid()
         rows = []
-        master_data = [
-            self.master.host,
-            self.master.port,
-            "MASTER",
-            get_server_state(self.master, self.master.host, self.pingtime,
-                             self.verbosity > 0),
-            have_gtid,
-            "OK" if rpl_health[0] else ", ".join(rpl_health[1]),
-        ]
-
-        m_status = self.master.get_status()
-        if len(m_status):
-            master_log, master_log_pos = m_status[0][0:2]
-        else:
-            master_log = None
-            master_log_pos = 0
-
         columns = []
         columns.extend(_HEALTH_COLS)
-        # Show additional details if verbosity turned on
         if self.verbosity > 0:
             columns.extend(_HEALTH_DETAIL_COLS)
-            master_data.extend([self.master.get_version(), master_log,
-                                master_log_pos, "", "", "", "", "", "", "", "",
-                                ""])
+        if self.master:
+            # Get master health
+            rpl_health = self.master.check_rpl_health()
+            self._report("# Getting health for master: %s:%s." %
+                         (self.master.host, self.master.port), logging.INFO,
+                         False)
+            have_gtid = self.master.supports_gtid()
+            master_data = [
+                self.master.host,
+                self.master.port,
+                "MASTER",
+                get_server_state(self.master, self.master.host, self.pingtime,
+                                 self.verbosity > 0),
+                have_gtid,
+                "OK" if rpl_health[0] else ", ".join(rpl_health[1]),
+            ]
 
-        rows.append(master_data)
+            m_status = self.master.get_status()
+            if len(m_status):
+                master_log, master_log_pos = m_status[0][0:2]
+            else:
+                master_log = None
+                master_log_pos = 0
+
+            # Show additional details if verbosity turned on
+            if self.verbosity > 0:
+                master_data.extend([self.master.get_version(), master_log,
+                                    master_log_pos, "", "", "", "", "", "",
+                                    "", "", ""])
+
+            rows.append(master_data)
+            if have_gtid == "ON":
+                master_gtids = self.master.exec_query(_GTID_EXECUTED)
+        else:
+            # No master makes these impossible to determine.
+            have_gtid = "OFF"
+            master_log = ""
+            master_log_pos = ""
 
-        slave_rows = []
         # Get the health of the slaves
-        if have_gtid == "ON":
-            master_gtids = self.master.exec_query(_GTID_EXECUTED)
+        slave_rows = []
         for slave_dict in self.slaves:
             host = slave_dict['host']
             port = slave_dict['port']
@@ -1301,11 +1337,13 @@ class Topology(Replication):
                     # Connection failed.
                     rpl_health = (False, ["Slave is not alive."])
                     slave = None
+            elif not self.master:
+                rpl_health = (False, ["No master specified."])
             elif not slave.is_configured_for_master(self.master):
                 rpl_health = (False, ["Slave is not connected to master."])
                 slave = None
 
-            if slave is not None:
+            if self.master and slave is not None:
                 rpl_health = slave.check_rpl_health(self.master,
                                                     master_log, master_log_pos,
                                                     self.max_delay,
@@ -1399,11 +1437,89 @@ class Topology(Replication):
 
         return (executed, purged, owned)
 
-    def check_privileges(self, failover=False):
+    def get_slaves_dict(self, skip_not_connected=True):
+        """Get a dictionary representation of the slaves in the topology.
+
+        This function converts the list of slaves in the topology to a
+        dictionary with all elements in the list, using 'host@port' as the
+        key for each element.
+
+        skip_not_connected[in]  Boolean value indicating if not available or
+                                not connected slaves should be skipped.
+                                By default 'True' (not available slaves are
+                                skipped).
+
+        Return a dictionary representation of the slaves in the
+        topology. Each element has a key with the format 'host@port' and
+        a dictionary value with the corresponding slave's data.
+        """
+        res = {}
+        for slave_dic in self.slaves:
+            slave = slave_dic['instance']
+            if skip_not_connected:
+                if slave and slave.is_alive():
+                    key = '{0}@{1}'.format(slave_dic['host'],
+                                           slave_dic['port'])
+                    res[key] = slave_dic
+            else:
+                key = '{0}@{1}'.format(slave_dic['host'], slave_dic['port'])
+                res[key] = slave_dic
+        return res
+
+    def slaves_gtid_subtract_executed(self, gtid_set, multithreading=False):
+        """Subtract GTID_EXECUTED from the given GTID set on all slaves.
+
+        Compute the difference between the given GTID set and the GTID_EXECUTED
+        set for each slave, providing the sets with the missing GTIDs from the
+        GTID_EXECUTED set that belong to the input GTID set.
+
+        gtid_set[in]        Input GTID set to find the missing element from
+                            the GTID_EXECUTED for all slaves.
+        multithreading[in]  Flag indicating if multithreading will be used,
+                            meaning that the operation will be performed
+                            concurrently on all slaves.
+                            By default True (concurrent execution).
+
+        Return a list of tuples with the result for each slave. Each tuple
+        contains the identification of the server (host and port) and a string
+        representing the set of GTIDs from the given set not in the
+        GTID_EXECUTED set of the corresponding slave.
+        """
+        if multithreading:
+            # Create a pool of threads to execute the method for each slave.
+            pool = ThreadPool(processes=len(self.slaves))
+            res_lst = []
+            for slave_dict in self.slaves:
+                slave = slave_dict['instance']
+                if slave:  # Skip non existing (not connected) slaves.
+                    thread_res = pool.apply_async(slave.gtid_subtract_executed,
+                                                  (gtid_set, ))
+                    res_lst.append((slave.host, slave.port, thread_res))
+            pool.close()
+            # Wait for all threads to finish here to avoid RuntimeErrors when
+            # waiting for the result of a thread that is already dead.
+            pool.join()
+            # Get the result from each slave and return the results.
+            res = []
+            for host, port, thread_res in res_lst:
+                res.append((host, port, thread_res.get()))
+            return res
+        else:
+            res = []
+            # Subtract gtid set on all slaves.
+            for slave_dict in self.slaves:
+                slave = slave_dict['instance']
+                if slave:  # Skip non existing (not connected) slaves.
+                    not_in_set = slave.gtid_subtract_executed(gtid_set)
+                    res.append((slave.host, slave.port, not_in_set))
+            return res
+
+    def check_privileges(self, failover=False, skip_master=False):
         """Check privileges for the master and all known servers
 
-        failover[in]   if True, check permissions for switchover and
-                       failover commands. Default is False.
+        failover[in]        if True, check permissions for switchover and
+                            failover commands. Default is False.
+        skip_master[in]     Skip the check for the master.
 
         Returns list - [(user, host)] if not enough permissions,
                        [] if no errors
@@ -1412,14 +1528,23 @@ class Topology(Replication):
         errors = []
 
         # Collect all users first.
-        if self.master is not None:
-            servers.append(self.master)
+        if skip_master:
             for slave_conn in self.slaves:
                 slave = slave_conn['instance']
                 # A slave instance is None if the connection failed during the
                 # creation of the topology. In this case ignore the slave.
                 if slave is not None:
                     servers.append(slave)
+        else:
+            if self.master is not None:
+                servers.append(self.master)
+                for slave_conn in self.slaves:
+                    slave = slave_conn['instance']
+                    # A slave instance is None if the connection failed during
+                    # the creation of the topology. In this case ignore the
+                    # slave.
+                    if slave is not None:
+                        servers.append(slave)
 
         # If candidates were specified, check those too.
         candidates = self.options.get("candidates", None)
@@ -1447,11 +1572,12 @@ class Topology(Replication):
                     not user_inst.has_privilege("*", "*", "RELOAD") or
                     not user_inst.has_privilege("*", "*", "DROP") or
                     not user_inst.has_privilege("*", "*", "CREATE") or
+                    not user_inst.has_privilege("*", "*", "INSERT") or
                     not user_inst.has_privilege("*", "*",
                                                 "REPLICATION SLAVE")):
                     errors.append((server.user, server.host, server.port,
                                    'SUPER, GRANT OPTION, REPLICATION SLAVE, '
-                                   'SELECT, RELOAD, DROP, CREATE'))
+                                   'SELECT, RELOAD, DROP, CREATE, INSERT'))
 
         # Disconnect if we connected to any candidates
         for slave in candidate_slaves:
@@ -1685,10 +1811,9 @@ class Topology(Replication):
 
         if self.verbose:
             self._report("# Creating replication user if it does not exist.")
-
         res = m_candidate.create_rpl_user(m_candidate.host,
                                           m_candidate.port,
-                                          user, passwd)
+                                          user, passwd, ssl=self.ssl)
         if not res[0]:
             print("# ERROR: {0}".format(res[1]))
             self._report(res[1], logging.CRITICAL, False)
@@ -1770,9 +1895,9 @@ class Topology(Replication):
         self.master = m_candidate
 
         # Remove slave from list of slaves
-        self._remove_slave({'host': m_candidate.host,
-                            'port': m_candidate.port,
-                            'instance': m_candidate})
+        self.remove_slave({'host': m_candidate.host,
+                           'port': m_candidate.port,
+                           'instance': m_candidate})
 
         # Make new master forget was an slave using slave methods
         s_candidate = self._change_role(m_candidate)
@@ -1789,6 +1914,24 @@ class Topology(Replication):
             'Master_Log_File': new_master_info[0],
             'Read_Master_Log_Pos': new_master_info[1],
         }
+
+        # Use the options SSL certificates if defined,
+        # else use the master SSL certificates if defined.
+        if self.ssl:
+            master_values['Master_SSL_Allowed'] = 1
+            if self.ssl_ca:
+                master_values['Master_SSL_CA_File'] = self.ssl_ca
+            if self.ssl_cert:
+                master_values['Master_SSL_Cert'] = self.ssl_cert
+            if self.ssl_key:
+                master_values['Master_SSL_Key'] = self.ssl_key
+
+        elif m_candidate.has_ssl:
+            master_values['Master_SSL_Allowed'] = 1
+            master_values['Master_SSL_CA_File'] = m_candidate.ssl_ca
+            master_values['Master_SSL_Cert'] = m_candidate.ssl_cert
+            master_values['Master_SSL_Key'] = m_candidate.ssl_key
+
         for slave_dict in self.slaves:
             slave = slave_dict['instance']
             # skip dead or zombie slaves
@@ -2020,7 +2163,8 @@ class Topology(Replication):
 
         # Need Master class instance to check master and replication user
         self.master = self._change_role(new_master, False)
-        res = self.master.create_rpl_user(host, port, user, passwd)
+        res = self.master.create_rpl_user(host, port, user, passwd,
+                                          ssl=self.ssl)
         if not res[0]:
             print("# ERROR: {0}".format(res[1]))
             self._report(res[1], logging.CRITICAL, False)
@@ -2034,7 +2178,7 @@ class Topology(Replication):
         self.run_cmd_on_slaves("stop", not self.verbose)
 
         # Take the new master out of the slaves list.
-        self._remove_slave(new_master_dict)
+        self.remove_slave(new_master_dict)
 
         self._report("# Switching slaves to new master.")
         for slave_dict in self.slaves:
