@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2011, 2013, Oracle and/or its affiliates. All rights
+# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights
 # reserved.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -92,6 +92,12 @@ class UtilTestError(UtilError):
         self.result = result
 
 
+class UtilDaemonError(UtilError):
+    """POSIX daemon error.
+    """
+    pass
+
+
 class FormatError(Error):
     """An entity was supplied in the wrong format."""
     pass
