@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,12 +34,18 @@ import re
 
 from mysql.utilities.command import dbimport
 from mysql.utilities.common.ip_parser import parse_connection
-from mysql.utilities.common.messages import WARN_OPT_ONLY_USED_WITH
-from mysql.utilities.common.options import (setup_common_options, add_engines,
-                                            add_skip_options, check_verbosity,
-                                            add_verbosity, check_skip_options,
-                                            add_format_option,
-                                            add_character_set_option)
+from mysql.utilities.common.messages import (WARN_OPT_ONLY_USED_WITH,
+                                             FILE_DOES_NOT_EXIST,
+                                             INSUFFICIENT_FILE_PERMISSIONS)
+from mysql.utilities.common.options import (add_character_set_option,
+                                            add_engines, add_format_option,
+                                            add_no_headers_option,
+                                            add_skip_options, add_verbosity,
+                                            check_skip_options,
+                                            check_verbosity,
+                                            setup_common_options,
+                                            get_ssl_dict,
+                                            check_password_security)
 from mysql.utilities.common.pattern_matching import REGEXP_QUALIFIED_OBJ_NAME
 from mysql.utilities.common.tools import (check_connector_python,
                                           print_elapsed_time)
@@ -95,10 +101,9 @@ if __name__ == '__main__':
                       dest="bulk_insert", default=False, help="use bulk "
                       "insert statements for data (default:False)")
 
-    # Header row
-    parser.add_option("-h", "--no-headers", action="store_true",
-                      dest="no_headers", default=False,
-                      help="files do not contain column headers")
+    # No header option
+    add_no_headers_option(parser, restricted_formats=['tab', 'csv'],
+                          help_msg="files do not contain column headers")
 
     # Dryrun mode
     parser.add_option("--dryrun", action="store_true", dest="dryrun",
@@ -154,6 +159,9 @@ if __name__ == '__main__':
     # Now we process the rest of the arguments.
     opt, args = parser.parse_args()
 
+    # Check security settings
+    check_password_security(opt, args)
+
     # Warn if quiet and verbosity are both specified
     check_verbosity(opt)
 
@@ -240,6 +248,7 @@ if __name__ == '__main__':
 
     # Parse server connection values
     try:
+        options.update(get_ssl_dict(opt))
         server_values = parse_connection(opt.server, None, options)
     except FormatError:
         _, err, _ = sys.exc_info()
@@ -270,6 +279,18 @@ if __name__ == '__main__':
     # Build list of files to import
     file_list = []
     for file_name in args:
+        # Test if is a file
+        if not os.path.isfile(file_name):
+            parser.error(FILE_DOES_NOT_EXIST.format(path=file_name))
+        # Test if is readable
+        try:
+            with open(file_name, "r"):
+                pass
+        except IOError:
+            parser.error(
+                INSUFFICIENT_FILE_PERMISSIONS.format(permissions="read",
+                                                     path=file_name))
+
         file_list.append(file_name)
 
     try:
