@@ -42,7 +42,9 @@ from mysql.utilities.common.messages import (PARSE_ERR_DB_OBJ_MISSING,
 from mysql.utilities.common.options import (add_difftype, add_verbosity,
                                             check_verbosity, add_changes_for,
                                             add_reverse, setup_common_options,
-                                            add_character_set_option)
+                                            add_character_set_option,
+                                            get_ssl_dict,
+                                            check_password_security)
 
 
 # Constants
@@ -60,21 +62,24 @@ if not check_connector_python():
 if __name__ == '__main__':
     # Setup the command parser
     parser = setup_common_options(os.path.basename(sys.argv[0]),
-                                  DESCRIPTION, USAGE, False, False)
+                                  DESCRIPTION, USAGE, False, False,
+                                  add_ssl=True)
 
     # Connection information for the source server
     parser.add_option("--server1", action="store", dest="server1",
                       type="string", default="root@localhost:3306",
                       help="connection information for first server in the "
                            "form: <user>[:<password>]@<host>[:<port>]"
-                           "[:<socket>] or <login-path>[:<port>][:<socket>].")
+                           "[:<socket>] or <login-path>[:<port>][:<socket>]"
+                           " or <config-path>[<[group]>].")
 
     # Connection information for the destination server
     parser.add_option("--server2", action="store", dest="server2",
                       type="string", default=None,
                       help="connection information for second server in the "
                            "form: <user>[:<password>]@<host>[:<port>]"
-                           "[:<socket>] or <login-path>[:<port>][:<socket>].")
+                           "[:<socket>] or <login-path>[:<port>][:<socket>]"
+                           " or <config-path>[<[group]>].")
 
     # Add character set option
     add_character_set_option(parser)
@@ -113,6 +118,9 @@ if __name__ == '__main__':
     # Now we process the rest of the arguments.
     opt, args = parser.parse_args()
 
+    # Check security settings
+    check_password_security(opt, args, "# ")
+
     # Warn if quiet and verbosity are both specified
     check_verbosity(opt)
 
@@ -130,6 +138,9 @@ if __name__ == '__main__':
         "charset": opt.charset,
     }
 
+    # add ssl options values.
+    options.update(get_ssl_dict(opt))
+
     # Parse server connection values
     try:
         server1_values = parse_connection(opt.server1, None, options)
