@@ -36,6 +36,7 @@ from mysql.utilities.common.ip_parser import parse_connection
 from mysql.utilities.common.tools import check_connector_python
 from mysql.utilities.common.options import (add_verbosity, add_format_option,
                                             setup_common_options,
+                                            add_ssl_options,
                                             check_password_security)
 
 
@@ -62,7 +63,8 @@ if __name__ == '__main__':
                       type="string", default="root@localhost:3306",
                       help="connection information for master server in the "
                            "form: <user>[:<password>]@<host>[:<port>]"
-                           "[:<socket>] or <login-path>[:<port>][:<socket>].")
+                           "[:<socket>] or <login-path>[:<port>][:<socket>]"
+                           " or <config-path>[<[group]>].")
 
     # Show graph option
     parser.add_option("-l", "--show-list", action="store_true",
@@ -102,6 +104,9 @@ if __name__ == '__main__':
                            "attempt. Valid with --prompt only.",
                       default=0)
 
+    # Add ssl options
+    add_ssl_options(parser)
+
     # Add verbosity mode and quiet option
     add_verbosity(parser, True)
 
